@@ -2,6 +2,7 @@
 
 /*
  Copyright (C) 2009 StatPro Italia srl
+ Copyright (C) 2019 Aprexo Limited
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -18,7 +19,7 @@
 */
 
 /*! \file version.hpp
-    \brief Version number
+    \brief Version number, and version of boost the library is compiled with
 */
 
 #ifndef quantlib_version_hpp
@@ -31,17 +32,26 @@
 
 //! version string
 #ifdef QL_DEBUG
-    #define QL_VERSION "1.14-debug"
+    #define QL_VERSION "1.18-debug"
 #else
-    #define QL_VERSION "1.14"
+    #define QL_VERSION "1.18"
 #endif
 
 //! version hexadecimal number
-#define QL_HEX_VERSION 0x011400f0
+#define QL_HEX_VERSION 0x011800f0
 //! version string for output lib name
-#define QL_LIB_VERSION "1_14"
+#define QL_LIB_VERSION "1_18"
 
 /*! @}  */
 
+namespace QuantLib {
+
+    /*! Returns the version of boost that the QuantLib library was built with
+        Use to check that client code is using a consistent version of boost.
+        Using QuantLib header files compiled with a different version of boost
+        than the library itself may result in undefined behaviour */
+    std::size_t compiledBoostVersion();
+
+}
 
 #endif
