@@ -6,6 +6,7 @@
  Copyright (C) 2006 Piter Dias
  Copyright (C) 2008 Charles Chongseok Hyun
  Copyright (C) 2015 Dmitri Nesteruk
+ Copyright (C) 2020 Piotr Siejda
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -62,6 +63,14 @@ void CalendarTest::testModifiedCalendars() {
     c1.addHoliday(d2);
 
     // test
+    std::set<Date> addedHolidays(c1.addedHolidays());
+    std::set<Date> removedHolidays(c1.removedHolidays());
+
+    QL_REQUIRE(addedHolidays.find(d1) == addedHolidays.end(), "did not expect to find date in addedHolidays");
+    QL_REQUIRE(addedHolidays.find(d2) != addedHolidays.end(), "expected to find date in addedHolidays");
+    QL_REQUIRE(removedHolidays.find(d1) != removedHolidays.end(), "expected to find date in removedHolidays");
+    QL_REQUIRE(removedHolidays.find(d2) == removedHolidays.end(), "did not expect to find date in removedHolidays");
+
     if (c1.isHoliday(d1))
         BOOST_FAIL(d1 << " still a holiday for original TARGET instance");
     if (c1.isBusinessDay(d2))
@@ -98,14 +107,24 @@ void CalendarTest::testJointCalendars() {
     Calendar c1 = TARGET(),
              c2 = UnitedKingdom(),
              c3 = UnitedStates(UnitedStates::NYSE),
-             c4 = Japan();
+             c4 = Japan(),
+             c5 = Germany();
+
+    std::vector<Calendar> calendar_vect;
+    calendar_vect.reserve( 5 );
+    calendar_vect.push_back(c1);
+    calendar_vect.push_back(c2);
+    calendar_vect.push_back(c3);
+    calendar_vect.push_back(c4);
+    calendar_vect.push_back(c5);
 
     Calendar c12h = JointCalendar(c1,c2,JoinHolidays),
              c12b = JointCalendar(c1,c2,JoinBusinessDays),
              c123h = JointCalendar(c1,c2,c3,JoinHolidays),
              c123b = JointCalendar(c1,c2,c3,JoinBusinessDays),
              c1234h = JointCalendar(c1,c2,c3,c4,JoinHolidays),
-             c1234b = JointCalendar(c1,c2,c3,c4,JoinBusinessDays);
+             c1234b = JointCalendar(c1,c2,c3,c4,JoinBusinessDays),
+             cvh = JointCalendar(calendar_vect,JoinHolidays);
 
     // test one year, starting today
     Date firstDate = Date::todaysDate(),
@@ -116,7 +135,8 @@ void CalendarTest::testJointCalendars() {
         bool b1 = c1.isBusinessDay(d),
              b2 = c2.isBusinessDay(d),
              b3 = c3.isBusinessDay(d),
-             b4 = c4.isBusinessDay(d);
+             b4 = c4.isBusinessDay(d),
+             b5 = c5.isBusinessDay(d);
 
         if ((b1 && b2) != c12h.isBusinessDay(d))
             BOOST_FAIL("At date " << d << ":\n"
@@ -154,6 +174,12 @@ void CalendarTest::testJointCalendars() {
                        << c1234b.name() << " (joining business days)\n"
                        << "    and its components");
 
+        if ((b1 && b2 && b3 && b4 && b5) != cvh.isBusinessDay(d))
+            BOOST_FAIL("At date " << d << ":\n"
+                       << "    inconsistency between joint calendar "
+                       << cvh.name() << " (joining holidays)\n"
+                       << "    and its components");
+
     }
 }
 
@@ -184,8 +210,8 @@ void CalendarTest::testUSSettlement() {
     expectedHol.push_back(Date(26,December,2005));
 
     Calendar c = UnitedStates(UnitedStates::Settlement);
-    std::vector<Date> hol = Calendar::holidayList(c, Date( 1, January, 2004),
-                                                     Date(31,December, 2005));
+    std::vector<Date> hol = c.holidayList(Date( 1, January, 2004),
+                                          Date(31,December, 2005));
     if (hol.size()!=expectedHol.size())
         BOOST_FAIL("there were " << expectedHol.size()
                    << " expected holidays, while there are " << hol.size()
@@ -207,8 +233,8 @@ void CalendarTest::testUSSettlement() {
     expectedHol.push_back(Date(23,November,1961));
     expectedHol.push_back(Date(25,December,1961));
 
-    hol = Calendar::holidayList(c, Date( 1, January, 1961),
-                                Date(31,December, 1961));
+    hol = c.holidayList(Date( 1, January, 1961),
+                        Date(31,December, 1961));
     if (hol.size()!=expectedHol.size())
         BOOST_FAIL("there were " << expectedHol.size()
                    << " expected holidays, while there are " << hol.size()
@@ -229,6 +255,7 @@ void CalendarTest::testUSGovernmentBondMarket() {
     expectedHol.push_back(Date(16,February,2004));
     expectedHol.push_back(Date(9,April,2004));
     expectedHol.push_back(Date(31,May,2004));
+    expectedHol.push_back(Date(11,June,2004)); // Reagan's funeral
     expectedHol.push_back(Date(5,July,2004));
     expectedHol.push_back(Date(6,September,2004));
     expectedHol.push_back(Date(11,October,2004));
@@ -237,8 +264,8 @@ void CalendarTest::testUSGovernmentBondMarket() {
     expectedHol.push_back(Date(24,December,2004));
 
     Calendar c = UnitedStates(UnitedStates::GovernmentBond);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2004),
-                                                     Date(31,December,2004));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2004),
+                                          Date(31,December,2004));
 
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
@@ -286,8 +313,8 @@ void CalendarTest::testUSNewYorkStockExchange() {
     expectedHol.push_back(Date(25,December,2006));
 
     Calendar c = UnitedStates(UnitedStates::NYSE);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2004),
-                                                     Date(31,December,2006));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2004),
+                                          Date(31,December,2006));
 
     Size i;
     for (i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
@@ -400,8 +427,8 @@ void CalendarTest::testTARGET() {
     expectedHol.push_back(Date(26,December,2006));
 
     Calendar c = TARGET();
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,1999),
-                                                     Date(31,December,2006));
+    std::vector<Date> hol = c.holidayList(Date(1,January,1999),
+                                          Date(31,December,2006));
 
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
@@ -427,17 +454,15 @@ void CalendarTest::testGermanyFrankfurt() {
     expectedHol.push_back(Date(24,December,2003));
     expectedHol.push_back(Date(25,December,2003));
     expectedHol.push_back(Date(26,December,2003));
-    expectedHol.push_back(Date(31,December,2003));
 
     expectedHol.push_back(Date(1,January,2004));
     expectedHol.push_back(Date(9,April,2004));
     expectedHol.push_back(Date(12,April,2004));
     expectedHol.push_back(Date(24,December,2004));
-    expectedHol.push_back(Date(31,December,2004));
 
     Calendar c = Germany(Germany::FrankfurtStockExchange);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2003),
-                                                     Date(31,December,2004));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2003),
+                                          Date(31,December,2004));
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
             BOOST_FAIL("expected holiday was " << expectedHol[i]
@@ -461,17 +486,15 @@ void CalendarTest::testGermanyEurex() {
     expectedHol.push_back(Date(24,December,2003));
     expectedHol.push_back(Date(25,December,2003));
     expectedHol.push_back(Date(26,December,2003));
-    expectedHol.push_back(Date(31,December,2003));
 
     expectedHol.push_back(Date(1,January,2004));
     expectedHol.push_back(Date(9,April,2004));
     expectedHol.push_back(Date(12,April,2004));
     expectedHol.push_back(Date(24,December,2004));
-    expectedHol.push_back(Date(31,December,2004));
 
     Calendar c = Germany(Germany::Eurex);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2003),
-                                                     Date(31,December,2004));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2003),
+                                          Date(31,December,2004));
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
             BOOST_FAIL("expected holiday was " << expectedHol[i]
@@ -495,17 +518,15 @@ void CalendarTest::testGermanyXetra() {
     expectedHol.push_back(Date(24,December,2003));
     expectedHol.push_back(Date(25,December,2003));
     expectedHol.push_back(Date(26,December,2003));
-    expectedHol.push_back(Date(31,December,2003));
 
     expectedHol.push_back(Date(1,January,2004));
     expectedHol.push_back(Date(9,April,2004));
     expectedHol.push_back(Date(12,April,2004));
     expectedHol.push_back(Date(24,December,2004));
-    expectedHol.push_back(Date(31,December,2004));
 
     Calendar c = Germany(Germany::Xetra);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2003),
-                                                     Date(31,December,2004));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2003),
+                                          Date(31,December,2004));
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
             BOOST_FAIL("expected holiday was " << expectedHol[i]
@@ -559,8 +580,8 @@ void CalendarTest::testUKSettlement() {
     expectedHol.push_back(Date(26,December,2007));
 
     Calendar c = UnitedKingdom(UnitedKingdom::Settlement);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2004),
-                                                     Date(31,December,2007));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2004),
+                                          Date(31,December,2007));
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
             BOOST_FAIL("expected holiday was " << expectedHol[i]
@@ -614,8 +635,8 @@ void CalendarTest::testUKExchange() {
     expectedHol.push_back(Date(26,December,2007));
 
     Calendar c = UnitedKingdom(UnitedKingdom::Exchange);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2004),
-                                                     Date(31,December,2007));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2004),
+                                          Date(31,December,2007));
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
             BOOST_FAIL("expected holiday was " << expectedHol[i]
@@ -669,8 +690,8 @@ void CalendarTest::testUKMetals() {
     expectedHol.push_back(Date(26,December,2007));
 
     Calendar c = UnitedKingdom(UnitedKingdom::Metals);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2004),
-                                                     Date(31,December,2007));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2004),
+                                          Date(31,December,2007));
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
             BOOST_FAIL("expected holiday was " << expectedHol[i]
@@ -714,8 +735,8 @@ void CalendarTest::testItalyExchange() {
     expectedHol.push_back(Date(31,December,2004));
 
     Calendar c = Italy(Italy::Exchange);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2002),
-                                                     Date(31,December,2004));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2002),
+                                          Date(31,December,2004));
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
             BOOST_FAIL("expected holiday was " << expectedHol[i]
@@ -1315,7 +1336,7 @@ void CalendarTest::testRussia()
   Calendar c = Russia(Russia::MOEX);
   
   
-  std::vector<Date> hol = Calendar::holidayList(c, 
+  std::vector<Date> hol = c.holidayList(
     Date(1, January, 2012),
     Date(31, December, 2016), // only dates for which calendars are available
     true); // include week-ends since lists are exhaustive
@@ -1362,8 +1383,8 @@ void CalendarTest::testBrazil() {
     expectedHol.push_back(Date(25,December,2006));
 
     Calendar c = Brazil();
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2005),
-                                                     Date(31,December,2006));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2005),
+                                          Date(31,December,2006));
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
             BOOST_FAIL("expected holiday was " << expectedHol[i]
@@ -1452,8 +1473,8 @@ void CalendarTest::testSouthKoreanSettlement() {
     expectedHol.push_back(Date(25,December,2007));
 
     Calendar c = SouthKorea(SouthKorea::Settlement);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2004),
-                                                     Date(31,December,2007));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2004),
+                                          Date(31,December,2007));
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
             BOOST_FAIL("expected holiday was " << expectedHol[i]
@@ -1545,8 +1566,8 @@ void CalendarTest::testKoreaStockExchange() {
     expectedHol.push_back(Date(31,December,2007));
 
     Calendar c = SouthKorea(SouthKorea::KRX);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1,January,2004),
-                                                     Date(31,December,2007));
+    std::vector<Date> hol = c.holidayList(Date(1,January,2004),
+                                          Date(31,December,2007));
 
     for (Size i=0; i<std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i]!=expectedHol[i])
@@ -1638,9 +1659,69 @@ void CalendarTest::testChinaSSE() {
     expectedHol.push_back(Date(5, Oct, 2017));
     expectedHol.push_back(Date(6, Oct, 2017));
 
+    // China Shanghai Securities Exchange holiday list in the year 2018
+    expectedHol.push_back(Date(1, Jan, 2018));
+    expectedHol.push_back(Date(15, Feb, 2018));
+    expectedHol.push_back(Date(16, Feb, 2018));
+    expectedHol.push_back(Date(19, Feb, 2018));
+    expectedHol.push_back(Date(20, Feb, 2018));
+    expectedHol.push_back(Date(21, Feb, 2018));
+    expectedHol.push_back(Date(5, April, 2018));
+    expectedHol.push_back(Date(6, April, 2018));
+    expectedHol.push_back(Date(30, April, 2018));
+    expectedHol.push_back(Date(1, May, 2018));
+    expectedHol.push_back(Date(18, June, 2018));
+    expectedHol.push_back(Date(24, September, 2018));
+    expectedHol.push_back(Date(1, Oct, 2018));
+    expectedHol.push_back(Date(2, Oct, 2018));
+    expectedHol.push_back(Date(3, Oct, 2018));
+    expectedHol.push_back(Date(4, Oct, 2018));
+    expectedHol.push_back(Date(5, Oct, 2018));
+    expectedHol.push_back(Date(31, December, 2018));
+
+    // China Shanghai Securities Exchange holiday list in the year 2019
+    expectedHol.push_back(Date(1, Jan, 2019));
+    expectedHol.push_back(Date(4, Feb, 2019));
+    expectedHol.push_back(Date(5, Feb, 2019));
+    expectedHol.push_back(Date(6, Feb, 2019));
+    expectedHol.push_back(Date(7, Feb, 2019));
+    expectedHol.push_back(Date(8, Feb, 2019));
+    expectedHol.push_back(Date(5, April, 2019));
+    expectedHol.push_back(Date(1, May, 2019));
+    expectedHol.push_back(Date(2, May, 2019));
+    expectedHol.push_back(Date(3, May, 2019));
+    expectedHol.push_back(Date(7, June, 2019));
+    expectedHol.push_back(Date(13, September, 2019));
+    expectedHol.push_back(Date(1, October, 2019));
+    expectedHol.push_back(Date(2, October, 2019));
+    expectedHol.push_back(Date(3, October, 2019));
+    expectedHol.push_back(Date(4, October, 2019));
+    expectedHol.push_back(Date(7, October, 2019));
+
+    // China Shanghai Securities Exchange holiday list in the year 2020
+    expectedHol.push_back(Date(1, Jan, 2020));
+    expectedHol.push_back(Date(24, Jan, 2020));
+    expectedHol.push_back(Date(27, Jan, 2020));
+    expectedHol.push_back(Date(28, Jan, 2020));
+    expectedHol.push_back(Date(29, Jan, 2020));
+    expectedHol.push_back(Date(30, Jan, 2020));
+    expectedHol.push_back(Date(31, Jan, 2020));
+    expectedHol.push_back(Date(6, April, 2020));
+    expectedHol.push_back(Date(1, May, 2020));
+    expectedHol.push_back(Date(4, May, 2020));
+    expectedHol.push_back(Date(5, May, 2020));
+    expectedHol.push_back(Date(25, June, 2020));
+    expectedHol.push_back(Date(26, June, 2020));
+    expectedHol.push_back(Date(1, October, 2020));
+    expectedHol.push_back(Date(2, October, 2020));
+    expectedHol.push_back(Date(5, October, 2020));
+    expectedHol.push_back(Date(6, October, 2020));
+    expectedHol.push_back(Date(7, October, 2020));
+    expectedHol.push_back(Date(8, October, 2020));
+
     Calendar c = China(China::SSE);
-    std::vector<Date> hol = Calendar::holidayList(c, Date(1, January, 2014),
-        Date(31, December, 2017));
+    std::vector<Date> hol = c.holidayList(Date(1, January, 2014),
+                                          Date(31, December, 2020));
 
     for (Size i = 0; i < std::min<Size>(hol.size(), expectedHol.size()); i++) {
         if (hol[i] != expectedHol[i])
@@ -1687,9 +1768,34 @@ void CalendarTest::testChinaIB() {
     expectedWorkingWeekEnds.push_back(Date(27, May, 2017));
     expectedWorkingWeekEnds.push_back(Date(30, Sep, 2017));
 
+    // China Inter Bank working weekends list in the year 2018
+    expectedWorkingWeekEnds.push_back(Date(11, Feb, 2018));
+    expectedWorkingWeekEnds.push_back(Date(24, Feb, 2018));
+    expectedWorkingWeekEnds.push_back(Date(8, April, 2018));
+    expectedWorkingWeekEnds.push_back(Date(28, April, 2018));
+    expectedWorkingWeekEnds.push_back(Date(29, Sep, 2018));
+    expectedWorkingWeekEnds.push_back(Date(30, Sep, 2018));
+    expectedWorkingWeekEnds.push_back(Date(29, December, 2018));
+
+    // China Inter Bank working weekends list in the year 2019
+    expectedWorkingWeekEnds.push_back(Date(2, Feb, 2019));
+    expectedWorkingWeekEnds.push_back(Date(3, Feb, 2019));
+    expectedWorkingWeekEnds.push_back(Date(28, April, 2019));
+    expectedWorkingWeekEnds.push_back(Date(5, May, 2019));
+    expectedWorkingWeekEnds.push_back(Date(29, September, 2019));
+    expectedWorkingWeekEnds.push_back(Date(12, October, 2019));
+
+    // China Inter Bank working weekends list in the year 2020
+    expectedWorkingWeekEnds.push_back(Date(19, January, 2020));
+    expectedWorkingWeekEnds.push_back(Date(26, April, 2020));
+    expectedWorkingWeekEnds.push_back(Date(9, May, 2020));
+    expectedWorkingWeekEnds.push_back(Date(28, June, 2020));
+    expectedWorkingWeekEnds.push_back(Date(27, September, 2020));
+    expectedWorkingWeekEnds.push_back(Date(10, October, 2020));
+
     Calendar c = China(China::IB);
     Date start(1, Jan, 2014);
-    Date end(31, Dec, 2017);
+    Date end(31, Dec, 2020);
 
     Size k = 0;
 
@@ -1738,22 +1844,27 @@ void CalendarTest::testEndOfMonth() {
 
 void CalendarTest::testBusinessDaysBetween() {
 
-    BOOST_TEST_MESSAGE("Testing calculation of business days between dates...");
+    BOOST_TEST_MESSAGE("Testing calculation of business days between dates..."); 
 
     std::vector<Date> testDates;
-    testDates.push_back(Date(1,February,2002));
-    testDates.push_back(Date(4,February,2002));
-    testDates.push_back(Date(16,May,2003));
-    testDates.push_back(Date(17,December,2003));
-    testDates.push_back(Date(17,December,2004));
-    testDates.push_back(Date(19,December,2005));
-    testDates.push_back(Date(2,January,2006));
-    testDates.push_back(Date(13,March,2006));
-    testDates.push_back(Date(15,May,2006));
-    testDates.push_back(Date(17,March,2006));
-    testDates.push_back(Date(15,May,2006));
-    testDates.push_back(Date(26,July,2006));
+    testDates.push_back(Date(1,February,2002)); //isBusinessDay = true
+    testDates.push_back(Date(4,February,2002)); //isBusinessDay = true
+    testDates.push_back(Date(16,May,2003)); //isBusinessDay = true
+    testDates.push_back(Date(17,December,2003)); //isBusinessDay = true
+    testDates.push_back(Date(17,December,2004)); //isBusinessDay = true
+    testDates.push_back(Date(19,December,2005)); //isBusinessDay = true
+    testDates.push_back(Date(2,January,2006)); //isBusinessDay = true
+    testDates.push_back(Date(13,March,2006)); //isBusinessDay = true
+    testDates.push_back(Date(15,May,2006)); //isBusinessDay = true
+    testDates.push_back(Date(17,March,2006)); //isBusinessDay = true
+    testDates.push_back(Date(15,May,2006)); //isBusinessDay = true
+    testDates.push_back(Date(26,July,2006)); //isBusinessDay = true
+    testDates.push_back(Date(26,July,2006)); //isBusinessDay = true
+    testDates.push_back(Date(27,July,2006)); //isBusinessDay = true
+	testDates.push_back(Date(29,July,2006)); //isBusinessDay = false
+    testDates.push_back(Date(29,July,2006)); //isBusinessDay = false
 
+	//default params: from date included, to excluded
     Date::serial_type expected[] = {
         1,
         321,
@@ -1765,19 +1876,107 @@ void CalendarTest::testBusinessDaysBetween() {
         42,
         -38,
         38,
-        51
+        51,
+		0,
+		1,
+		2,
+		0
     };
 
+	//exclude from, include to
+	Date::serial_type expected_include_to[] = {
+		1,
+		321,
+		152,
+		251,
+		252,
+		10,
+		48,
+		42,
+		-38,
+		38,
+		51,
+		0,
+		1,
+		1,
+		0
+	};
+
+	//include both from and to
+	Date::serial_type expected_include_all[] = {
+		2,
+		322,
+		153,
+		252,
+        253,
+		11,
+		49,
+		43,
+		-39,
+		39,
+		52,
+		1,
+		2,
+		2,
+		0
+	};
+
+	//exclude both from and to
+	Date::serial_type expected_exclude_all[] = {
+		0,
+		320,
+		151,
+		250,
+		251,
+		9,
+		47,
+		41,
+		-37,
+		37,
+		50,
+		0,
+		0,
+		1,
+		0
+	};
+
     Calendar calendar = Brazil();
 
     for (Size i=1; i<testDates.size(); i++) {
         Integer calculated = calendar.businessDaysBetween(testDates[i-1],
-                                                          testDates[i]);
+                                                          testDates[i], true, false);
         if (calculated != expected[i-1]) {
-            BOOST_ERROR("from " << testDates[i-1]
-                        << " to " << testDates[i] << ":\n"
+            BOOST_ERROR("from " << testDates[i-1] << " included"
+                        << " to " << testDates[i] << " excluded:\n"
                         << "    calculated: " << calculated << "\n"
                         << "    expected:   " << expected[i-1]);
+        }
+
+		calculated = calendar.businessDaysBetween(testDates[i-1],
+														  testDates[i], false, true);
+		if (calculated != expected_include_to[i-1]) {
+            BOOST_ERROR("from " << testDates[i-1] << " excluded"
+                        << " to " << testDates[i] << " included:\n"
+                        << "    calculated: " << calculated << "\n"
+                        << "    expected:   " << expected_include_to[i-1]);
+        }
+
+		calculated = calendar.businessDaysBetween(testDates[i-1],
+                                                          testDates[i], true, true);
+		if (calculated != expected_include_all[i-1]) {
+        	BOOST_ERROR("from " << testDates[i-1] << " included"
+                        << " to " << testDates[i] << " included:\n"
+                        << "    calculated: " << calculated << "\n"
+                        << "    expected:   " << expected_include_all[i-1]);
+		}
+
+		calculated = calendar.businessDaysBetween(testDates[i-1],
+                                                                  testDates[i], false, false);
+        if (calculated != expected_exclude_all[i-1]) {
+            BOOST_ERROR("from " << testDates[i-1] << " excluded"
+                        << " to " << testDates[i] << " excluded:\n"
+                        << "    calculated: " << calculated << "\n"
+                        << "    expected:   " << expected_exclude_all[i-1]);
         }
     }
 }
@@ -1897,6 +2096,124 @@ void CalendarTest::testBespokeCalendars() {
 
 }
 
+void CalendarTest::testIntradayAddHolidays() {
+#ifdef QL_HIGH_RESOLUTION_DATE
+    BOOST_TEST_MESSAGE("Testing addHolidays with enable-intraday...");
+
+    // test cases taken from testModifiedCalendars
+
+    Calendar c1 = TARGET();
+    Calendar c2 = UnitedStates(UnitedStates::NYSE);
+    Date d1(1,May,2004);      // holiday for both calendars
+    Date d2(26,April,2004, 0, 0, 1, 1);   // business day
+
+    Date d1Mock(1,May,2004, 1, 1, 0, 0);      // holiday for both calendars
+    Date d2Mock(26,April,2004);   // business day
+
+    // this works anyhow because implementation uses day() month() etc
+    QL_REQUIRE(c1.isHoliday(d1), "wrong assumption---correct the test");
+    QL_REQUIRE(c1.isBusinessDay(d2), "wrong assumption---correct the test");
+
+    QL_REQUIRE(c2.isHoliday(d1), "wrong assumption---correct the test");
+    QL_REQUIRE(c2.isBusinessDay(d2), "wrong assumption---correct the test");
+
+    // now with different hourly/min/sec
+    QL_REQUIRE(c1.isHoliday(d1Mock), "wrong assumption---correct the test");
+    QL_REQUIRE(c1.isBusinessDay(d2Mock), "wrong assumption---correct the test");
+
+    QL_REQUIRE(c2.isHoliday(d1Mock), "wrong assumption---correct the test");
+    QL_REQUIRE(c2.isBusinessDay(d2Mock), "wrong assumption---correct the test");
+
+
+    // modify the TARGET calendar
+    c1.removeHoliday(d1);
+    c1.addHoliday(d2);
+
+    // test
+    if (c1.isHoliday(d1))
+        BOOST_FAIL(d1 << " still a holiday for original TARGET instance");
+    if (c1.isBusinessDay(d2))
+        BOOST_FAIL(d2 << " still a business day for original TARGET instance");
+
+    if (c1.isHoliday(d1Mock))
+        BOOST_FAIL(d1Mock << " still a holiday for original TARGET instance" <<
+                    " and different hours/min/secs");
+    if (c1.isBusinessDay(d2Mock))
+        BOOST_FAIL(d2Mock << " still a business day for generic TARGET instance" <<
+                    " and different hours/min/secs");
+
+    // any instance of TARGET should be modified...
+    Calendar c3 = TARGET();
+    if (c3.isHoliday(d1))
+        BOOST_FAIL(d1 << " still a holiday for generic TARGET instance");
+    if (c3.isBusinessDay(d2))
+        BOOST_FAIL(d2 << " still a business day for generic TARGET instance");
+
+    if (c3.isHoliday(d1Mock))
+        BOOST_FAIL(d1Mock << " still a holiday for original TARGET instance" <<
+                    " and different hours/min/secs");
+    if (c3.isBusinessDay(d2Mock))
+        BOOST_FAIL(d2Mock << " still a business day for generic TARGET instance" <<
+                    " and different hours/min/secs");
+
+    // ...but not other calendars
+    if (c2.isBusinessDay(d1))
+        BOOST_FAIL(d1 << " business day for New York");
+    if (c2.isHoliday(d2))
+        BOOST_FAIL(d2 << " holiday for New York");
+
+    if (c2.isBusinessDay(d1Mock))
+        BOOST_FAIL(d1Mock << " business day for New York" <<
+                    " and different hours/min/secs");
+    if (c2.isHoliday(d2Mock))
+        BOOST_FAIL(d2Mock << " holiday for New York" <<
+                    " and different hours/min/secs");
+
+    // restore original holiday set---test the other way around
+    c3.addHoliday(d1Mock);
+    c3.removeHoliday(d2Mock);
+
+    if (c1.isBusinessDay(d1))
+        BOOST_FAIL(d1 << " still a business day");
+    if (c1.isHoliday(d2))
+        BOOST_FAIL(d2 << " still a holiday");
+
+    if (c1.isBusinessDay(d1Mock))
+        BOOST_FAIL(d1Mock << " still a business day" <<
+                    " and different hours/min/secs");
+    if (c1.isHoliday(d2Mock))
+        BOOST_FAIL(d2Mock << " still a holiday and different hours/min/secs");
+
+#endif
+}
+
+void CalendarTest::testDayLists() {
+
+    BOOST_TEST_MESSAGE("Testing holidayList and businessDaysList...");
+    Calendar germany = Germany();
+    Date firstDate = Settings::instance().evaluationDate(),
+         endDate = firstDate + 1*Years;
+
+    std::vector<Date> holidays = germany.holidayList(firstDate, endDate, true);
+    std::vector<Date> businessDays = germany.businessDayList(firstDate, endDate);
+
+    std::vector<Date>::iterator it_holidays = holidays.begin(),
+                                it_businessDays = businessDays.begin();
+    for (Date d = firstDate; d < endDate; d++) {
+        if(d == *it_holidays && d == *it_businessDays) {
+            BOOST_FAIL("Date " << d << "is both holiday and business day.");
+            ++it_holidays;
+            ++it_businessDays;
+        } else if (d == *it_holidays){
+            ++it_holidays;
+        } else if (d == *it_businessDays) {
+            ++it_businessDays;
+        } else {
+            BOOST_FAIL( "Date " << d << "is neither holiday nor business day.");
+        }
+    }
+
+}
 
 test_suite* CalendarTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("Calendar tests");
@@ -1935,6 +2252,9 @@ test_suite* CalendarTest::suite() {
     suite->add(QUANTLIB_TEST_CASE(&CalendarTest::testEndOfMonth));
     suite->add(QUANTLIB_TEST_CASE(&CalendarTest::testBusinessDaysBetween));
 
+    suite->add(QUANTLIB_TEST_CASE(&CalendarTest::testIntradayAddHolidays));
+    suite->add(QUANTLIB_TEST_CASE(&CalendarTest::testDayLists));
+
     return suite;
 }
 
