@@ -3,6 +3,7 @@
 /*
  Copyright (C) 2003 RiskMap srl
  Copyright (C) 2004, 2005, 2006, 2007, 2008 StatPro Italia srl
+ Copyright (C) 2019 Wojciech Ślusarski
 
  This file is part of QuantLib, a free-software/open-source library
  for financial quantitative analysts and developers - http://quantlib.org/
@@ -24,8 +25,10 @@
 #include <ql/instruments/vanillaswap.hpp>
 #include <ql/cashflows/cashflowvectors.hpp>
 #include <ql/termstructures/yield/flatforward.hpp>
+#include <ql/termstructures/yield/zerospreadedtermstructure.hpp>
 #include <ql/indexes/ibor/euribor.hpp>
 #include <ql/pricingengines/capfloor/blackcapfloorengine.hpp>
+#include <ql/pricingengines/capfloor/bacheliercapfloorengine.hpp>
 #include <ql/pricingengines/swap/discountingswapengine.hpp>
 #include <ql/models/marketmodels/models/flatvol.hpp>
 #include <ql/models/marketmodels/correlations/expcorrelations.hpp>
@@ -41,7 +44,7 @@
 using namespace QuantLib;
 using namespace boost::unit_test_framework;
 
-namespace {
+namespace capfloor_test {
 
     struct CommonVars {
         // common data
@@ -49,7 +52,7 @@ namespace {
         std::vector<Real> nominals;
         BusinessDayConvention convention;
         Frequency frequency;
-        boost::shared_ptr<IborIndex> index;
+        ext::shared_ptr<IborIndex> index;
         Calendar calendar;
         Natural fixingDays;
         RelinkableHandle<YieldTermStructure> termStructure;
@@ -62,7 +65,7 @@ namespace {
         CommonVars()
         : nominals(1,100) {
             frequency = Semiannual;
-            index = boost::shared_ptr<IborIndex>(new Euribor6M(termStructure));
+            index = ext::shared_ptr<IborIndex>(new Euribor6M(termStructure));
             calendar = index->fixingCalendar();
             convention = ModifiedFollowing;
             Date today = calendar.adjust(Date::todaysDate());
@@ -87,31 +90,43 @@ namespace {
                 .withFixingDays(fixingDays);
         }
 
-        boost::shared_ptr<PricingEngine> makeEngine(Volatility volatility) {
-            Handle<Quote> vol(boost::shared_ptr<Quote>(
+        ext::shared_ptr<PricingEngine> makeEngine(Volatility volatility) {
+            Handle<Quote> vol(ext::shared_ptr<Quote>(
                                                 new SimpleQuote(volatility)));
-            return boost::shared_ptr<PricingEngine>(
+            return ext::shared_ptr<PricingEngine>(
                                 new BlackCapFloorEngine(termStructure, vol));
         }
 
-        boost::shared_ptr<CapFloor> makeCapFloor(CapFloor::Type type,
+        ext::shared_ptr<PricingEngine> makeBachelierEngine(Volatility volatility) {
+            Handle<Quote> vol(ext::shared_ptr<Quote>(
+                                                new SimpleQuote(volatility)));
+            return ext::shared_ptr<PricingEngine>(
+                                new BachelierCapFloorEngine(termStructure, vol));
+        }
+
+        ext::shared_ptr<CapFloor> makeCapFloor(CapFloor::Type type,
                                                  const Leg& leg,
                                                  Rate strike,
-                                                 Volatility volatility) {
-            boost::shared_ptr<CapFloor> result;
+                                                 Volatility volatility,
+                                                 bool isLogNormal = true) {
+            ext::shared_ptr<CapFloor> result;
             switch (type) {
               case CapFloor::Cap:
-                result = boost::shared_ptr<CapFloor>(
+                result = ext::shared_ptr<CapFloor>(
                                   new Cap(leg, std::vector<Rate>(1, strike)));
                 break;
               case CapFloor::Floor:
-                result = boost::shared_ptr<CapFloor>(
+                result = ext::shared_ptr<CapFloor>(
                                 new Floor(leg, std::vector<Rate>(1, strike)));
                 break;
               default:
                 QL_FAIL("unknown cap/floor type");
             }
-            result->setPricingEngine(makeEngine(volatility));
+            if(isLogNormal){
+                result->setPricingEngine(makeEngine(volatility));
+            } else {
+                result->setPricingEngine(makeBachelierEngine(volatility));
+            }
             return result;
         }
 
@@ -141,6 +156,8 @@ void CapFloorTest::testVega() {
 
     BOOST_TEST_MESSAGE("Testing cap/floor vega...");
 
+    using namespace capfloor_test;
+
     CommonVars vars;
 
     Integer lengths[] = { 1, 2, 3, 4, 5, 6, 7, 10, 15, 20, 30 };
@@ -157,13 +174,13 @@ void CapFloorTest::testVega() {
             for (Size k=0; k<LENGTH(strikes); k++) {
                 for (Size h=0; h<LENGTH(types); h++) {
                     Leg leg = vars.makeLeg(startDate, lengths[i]);
-                    boost::shared_ptr<CapFloor> capFloor =
+                    ext::shared_ptr<CapFloor> capFloor =
                         vars.makeCapFloor(types[h],leg,
                                           strikes[k],vols[j]);
-                    boost::shared_ptr<CapFloor> shiftedCapFloor2 =
+                    ext::shared_ptr<CapFloor> shiftedCapFloor2 =
                         vars.makeCapFloor(types[h],leg,
                                           strikes[k],vols[j]+shift);
-                     boost::shared_ptr<CapFloor> shiftedCapFloor1 =
+                     ext::shared_ptr<CapFloor> shiftedCapFloor1 =
                         vars.makeCapFloor(types[h],leg,
                                           strikes[k],vols[j]-shift);
                     Real value1 = shiftedCapFloor1->NPV();
@@ -196,6 +213,8 @@ void CapFloorTest::testStrikeDependency() {
 
     BOOST_TEST_MESSAGE("Testing cap/floor dependency on strike...");
 
+    using namespace capfloor_test;
+
     CommonVars vars;
 
     Integer lengths[] = { 1, 2, 3, 5, 7, 10, 15, 20 };
@@ -210,11 +229,11 @@ void CapFloorTest::testStrikeDependency() {
             std::vector<Real> cap_values, floor_values;
             for (Size k=0; k<LENGTH(strikes); k++) {
                 Leg leg = vars.makeLeg(startDate,lengths[i]);
-                boost::shared_ptr<Instrument> cap =
+                ext::shared_ptr<Instrument> cap =
                     vars.makeCapFloor(CapFloor::Cap,leg,
                                       strikes[k],vols[j]);
                 cap_values.push_back(cap->NPV());
-                boost::shared_ptr<Instrument> floor =
+                ext::shared_ptr<Instrument> floor =
                     vars.makeCapFloor(CapFloor::Floor,leg,
                                       strikes[k],vols[j]);
                 floor_values.push_back(floor->NPV());
@@ -258,6 +277,8 @@ void CapFloorTest::testConsistency() {
 
     BOOST_TEST_MESSAGE("Testing consistency between cap, floor and collar...");
 
+    using namespace capfloor_test;
+
     CommonVars vars;
 
     Integer lengths[] = { 1, 2, 3, 5, 7, 10, 15, 20 };
@@ -273,10 +294,10 @@ void CapFloorTest::testConsistency() {
           for (Size l=0; l<LENGTH(vols); l++) {
 
               Leg leg = vars.makeLeg(startDate,lengths[i]);
-              boost::shared_ptr<CapFloor> cap =
+              ext::shared_ptr<CapFloor> cap =
                   vars.makeCapFloor(CapFloor::Cap,leg,
                                     cap_rates[j],vols[l]);
-              boost::shared_ptr<CapFloor> floor =
+              ext::shared_ptr<CapFloor> floor =
                   vars.makeCapFloor(CapFloor::Floor,leg,
                                     floor_rates[k],vols[l]);
               Collar collar(leg,std::vector<Rate>(1,cap_rates[j]),
@@ -297,7 +318,7 @@ void CapFloorTest::testConsistency() {
 
               // test re-composition by optionlets, N.B. two per year
               Real capletsNPV = 0.0;
-              std::vector<boost::shared_ptr<CapFloor> > caplets;
+              std::vector<ext::shared_ptr<CapFloor> > caplets;
               for (Integer m=0; m<lengths[i]*2; m++) {
                 caplets.push_back(cap->optionlet(m));
                 caplets[m]->setPricingEngine(vars.makeEngine(vols[l]));
@@ -317,7 +338,7 @@ void CapFloorTest::testConsistency() {
               }
 
               Real floorletsNPV = 0.0;
-              std::vector<boost::shared_ptr<CapFloor> > floorlets;
+              std::vector<ext::shared_ptr<CapFloor> > floorlets;
               for (Integer m=0; m<lengths[i]*2; m++) {
                 floorlets.push_back(floor->optionlet(m));
                 floorlets[m]->setPricingEngine(vars.makeEngine(vols[l]));
@@ -337,7 +358,7 @@ void CapFloorTest::testConsistency() {
               }
 
               Real collarletsNPV = 0.0;
-              std::vector<boost::shared_ptr<CapFloor> > collarlets;
+              std::vector<ext::shared_ptr<CapFloor> > collarlets;
               for (Integer m=0; m<lengths[i]*2; m++) {
                 collarlets.push_back(collar.optionlet(m));
                 collarlets[m]->setPricingEngine(vars.makeEngine(vols[l]));
@@ -372,6 +393,8 @@ void CapFloorTest::testParity() {
 
     BOOST_TEST_MESSAGE("Testing cap/floor parity...");
 
+    using namespace capfloor_test;
+
     CommonVars vars;
 
     Integer lengths[] = { 1, 2, 3, 5, 7, 10, 15, 20 };
@@ -385,10 +408,10 @@ void CapFloorTest::testParity() {
         for (Size k=0; k<LENGTH(vols); k++) {
 
             Leg leg = vars.makeLeg(startDate,lengths[i]);
-            boost::shared_ptr<Instrument> cap =
+            ext::shared_ptr<Instrument> cap =
                 vars.makeCapFloor(CapFloor::Cap,leg,
                                   strikes[j],vols[k]);
-            boost::shared_ptr<Instrument> floor =
+            ext::shared_ptr<Instrument> floor =
                 vars.makeCapFloor(CapFloor::Floor,leg,
                                   strikes[j],vols[k]);
             Date maturity = vars.calendar.advance(startDate,lengths[i],Years,
@@ -401,7 +424,7 @@ void CapFloorTest::testParity() {
                              schedule, strikes[j], vars.index->dayCounter(),
                              schedule, vars.index, 0.0,
                              vars.index->dayCounter());
-            swap.setPricingEngine(boost::shared_ptr<PricingEngine>(
+            swap.setPricingEngine(ext::shared_ptr<PricingEngine>(
                               new DiscountingSwapEngine(vars.termStructure)));
             // FLOATING_POINT_EXCEPTION
             if (std::fabs((cap->NPV()-floor->NPV()) - swap.NPV()) > 1.0e-10) {
@@ -423,6 +446,8 @@ void CapFloorTest::testATMRate() {
 
     BOOST_TEST_MESSAGE("Testing cap/floor ATM rate...");
 
+    using namespace capfloor_test;
+
     CommonVars vars;
 
     Integer lengths[] = { 1, 2, 3, 5, 7, 10, 15, 20 };
@@ -442,9 +467,9 @@ void CapFloorTest::testATMRate() {
 
         for (Size j=0; j<LENGTH(strikes); j++) {
             for (Size k=0; k<LENGTH(vols); k++) {
-                boost::shared_ptr<CapFloor> cap =
+                ext::shared_ptr<CapFloor> cap =
                     vars.makeCapFloor(CapFloor::Cap, leg, strikes[j],vols[k]);
-                boost::shared_ptr<CapFloor> floor =
+                ext::shared_ptr<CapFloor> floor =
                     vars.makeCapFloor(CapFloor::Floor, leg, strikes[j],vols[k]);
                 Rate capATMRate = cap->atmRate(**vars.termStructure);
                 Rate floorATMRate = floor->atmRate(**vars.termStructure);
@@ -464,7 +489,7 @@ void CapFloorTest::testATMRate() {
                                  vars.index->dayCounter(),
                                  schedule, vars.index, 0.0,
                                  vars.index->dayCounter());
-                swap.setPricingEngine(boost::shared_ptr<PricingEngine>(
+                swap.setPricingEngine(ext::shared_ptr<PricingEngine>(
                               new DiscountingSwapEngine(vars.termStructure)));
                 Real swapNPV = swap.NPV();
                 if (!checkAbsError(swapNPV, 0, 1.0e-10))
@@ -485,6 +510,8 @@ void CapFloorTest::testImpliedVolatility() {
 
     BOOST_TEST_MESSAGE("Testing implied term volatility for cap and floor...");
 
+    using namespace capfloor_test;
+
     CommonVars vars;
 
     Size maxEvaluations = 100;
@@ -504,7 +531,7 @@ void CapFloorTest::testImpliedVolatility() {
         for (Size i=0; i<LENGTH(types); i++) {
             for (Size j=0; j<LENGTH(strikes); j++) {
 
-                boost::shared_ptr<CapFloor> capfloor =
+                ext::shared_ptr<CapFloor> capfloor =
                     vars.makeCapFloor(types[i], leg, strikes[j], 0.0);
 
                 for (Size n=0; n<LENGTH(rRates); n++) {
@@ -573,6 +600,8 @@ void CapFloorTest::testCachedValue() {
 
     BOOST_TEST_MESSAGE("Testing Black cap/floor price against cached values...");
 
+    using namespace capfloor_test;
+
     CommonVars vars;
 
     Date cachedToday(14,March,2002),
@@ -581,19 +610,22 @@ void CapFloorTest::testCachedValue() {
     vars.termStructure.linkTo(flatRate(cachedSettlement, 0.05, Actual360()));
     Date startDate = vars.termStructure->referenceDate();
     Leg leg = vars.makeLeg(startDate,20);
-    boost::shared_ptr<Instrument> cap = vars.makeCapFloor(CapFloor::Cap,leg,
+    ext::shared_ptr<Instrument> cap = vars.makeCapFloor(CapFloor::Cap,leg,
                                                           0.07,0.20);
-    boost::shared_ptr<Instrument> floor = vars.makeCapFloor(CapFloor::Floor,leg,
+    ext::shared_ptr<Instrument> floor = vars.makeCapFloor(CapFloor::Floor,leg,
                                                             0.03,0.20);
-#ifndef QL_USE_INDEXED_COUPON
-    // par coupon price
-    Real cachedCapNPV   = 6.87570026732,
-         cachedFloorNPV = 2.65812927959;
-#else
-    // index fixing price
-    Real cachedCapNPV   = 6.87630307745,
-         cachedFloorNPV = 2.65796764715;
-#endif
+
+    Real cachedCapNPV, cachedFloorNPV ;
+    if (!IborCoupon::usingAtParCoupons()) {
+        // index fixing price
+        cachedCapNPV   = 6.87630307745,
+        cachedFloorNPV = 2.65796764715;
+    } else {
+        // par coupon price
+        cachedCapNPV   = 6.87570026732;
+        cachedFloorNPV = 2.65812927959;
+    }
+
     // test Black cap price against cached value
     if (std::fabs(cap->NPV()-cachedCapNPV) > 1.0e-11)
         BOOST_ERROR(
@@ -610,6 +642,323 @@ void CapFloorTest::testCachedValue() {
             << "    expected:   " << cachedFloorNPV);
 }
 
+void CapFloorTest::testCachedValueFromOptionLets() {
+
+    BOOST_TEST_MESSAGE("Testing Black cap/floor price as a sum of optionlets prices against cached values...");
+
+    using namespace capfloor_test;
+
+    CommonVars vars;
+
+    Date cachedToday(14,March,2002),
+         cachedSettlement(18,March,2002);
+    Settings::instance().evaluationDate() = cachedToday;
+    ext::shared_ptr<YieldTermStructure> baseCurve = flatRate(cachedSettlement, 
+                                                             0.05, Actual360());                                              
+    vars.termStructure.linkTo(baseCurve);
+    Date startDate = vars.termStructure->referenceDate();
+    Leg leg = vars.makeLeg(startDate,20);  
+
+    ext::shared_ptr<Instrument> cap = vars.makeCapFloor(CapFloor::Cap,leg,
+                                                          0.07,0.20);
+    ext::shared_ptr<Instrument> floor = vars.makeCapFloor(CapFloor::Floor,leg,
+                                                            0.03,0.20);
+    Real calculatedCapletsNPV = 0.0,
+         calculatedFloorletsNPV = 0.0;
+
+    Real cachedCapNPV, cachedFloorNPV;
+    if (IborCoupon::usingAtParCoupons()) {
+        cachedCapNPV = 6.87570026732;
+        cachedFloorNPV = 2.65812927959;
+    } else {
+        cachedCapNPV = 6.87630307745;
+        cachedFloorNPV = 2.65796764715;
+    }
+
+    // test Black floor price against cached value
+    std::vector<Real> capletPrices;
+    std::vector<Real> floorletPrices;
+    
+    capletPrices = cap->result<std::vector<Real> >("optionletsPrice");
+    floorletPrices = floor->result<std::vector<Real> >("optionletsPrice");
+    
+    if (capletPrices.size() != 40)
+        BOOST_ERROR(
+            "failed to produce prices for all caplets:\n"
+            << "    calculated: " << capletPrices.size() << " caplet prices\n"
+            << "    expected:   " << 40);
+
+    for (Size n=0; n<capletPrices.size(); n++){
+        calculatedCapletsNPV += capletPrices[n];
+    }
+
+    for (Size n=0; n<floorletPrices.size(); n++){
+        calculatedFloorletsNPV += floorletPrices[n];
+    }
+
+    if (std::fabs(calculatedCapletsNPV-cachedCapNPV) > 1.0e-11)
+        BOOST_ERROR(
+            "failed to reproduce cached cap value from its caplets' values:\n"
+            << std::setprecision(12)
+            << "    calculated: " << calculatedCapletsNPV << "\n"
+            << "    expected:   " << cachedCapNPV);
+    // test Black floor price against cached value
+    if (std::fabs(calculatedFloorletsNPV-cachedFloorNPV) > 1.0e-11)
+        BOOST_ERROR(
+            "failed to reproduce cached floor value from its floorlets' values:\n"
+            << std::setprecision(12)
+            << "    calculated: " << calculatedFloorletsNPV << "\n"
+            << "    expected:   " << cachedFloorNPV);
+}
+
+void CapFloorTest::testOptionLetsDelta() {
+
+    BOOST_TEST_MESSAGE("Testing Black caplet/floorlet delta coefficients against finite difference values...");
+
+    using namespace capfloor_test;
+
+    CommonVars vars;
+
+    Date cachedToday(14,March,2002),
+         cachedSettlement(18,March,2002);
+    Settings::instance().evaluationDate() = cachedToday;
+    ext::shared_ptr<YieldTermStructure> baseCurve = flatRate(cachedSettlement, 
+                                                             0.05, Actual360());
+    RelinkableHandle<YieldTermStructure> baseCurveHandle(baseCurve);
+
+    // Define spreaded curve with eps as spread used for FD sensitivities
+    Real eps = 1.0e-6;
+    ext::shared_ptr<SimpleQuote> spread(new SimpleQuote(0.0));
+    ext::shared_ptr<YieldTermStructure> spreadCurve(new ZeroSpreadedTermStructure(
+                                                            baseCurveHandle,
+                                                            Handle<Quote>(spread),
+                                                            Continuous,
+                                                            Annual,
+                                                            Actual360()));                                               
+    vars.termStructure.linkTo(spreadCurve);
+    Date startDate = vars.termStructure->referenceDate();
+    Leg leg = vars.makeLeg(startDate,20);  
+
+    ext::shared_ptr<CapFloor> cap = vars.makeCapFloor(CapFloor::Cap,leg,
+                                                          0.05,0.20);
+    ext::shared_ptr<CapFloor> floor = vars.makeCapFloor(CapFloor::Floor,leg,
+                                                            0.05,0.20);
+
+    
+    //so far tests pass, now try to get additional results and it will fail
+    Size capletsNum = cap->capRates().size();
+    std::vector<Real> capletUpPrices, 
+                      capletDownPrices,
+                      capletAnalyticDelta,
+                      capletDiscountFactorsUp,
+                      capletDiscountFactorsDown,
+                      capletForwardsUp,
+                      capletForwardsDown,
+                      capletFDDelta(capletsNum, 0.0); 
+    Size floorletNum = floor->floorRates().size();
+    std::vector<Real> floorletUpPrices, 
+                      floorletDownPrices,
+                      floorletAnalyticDelta,
+                      floorletDiscountFactorsUp,
+                      floorletDiscountFactorsDown,
+                      floorletForwardsUp,
+                      floorletForwardsDown,
+                      floorletFDDelta(floorletNum, 0.0);
+    
+    capletAnalyticDelta = cap->result<std::vector<Real> >("optionletsDelta");
+    floorletAnalyticDelta = floor->result<std::vector<Real> >("optionletsDelta");
+    
+    spread->setValue(eps);
+    capletUpPrices = cap->result<std::vector<Real> >("optionletsPrice");
+    floorletUpPrices = floor->result<std::vector<Real> >("optionletsPrice");
+    capletDiscountFactorsUp = cap->result<std::vector<Real> >("optionletsDiscountFactor");
+    floorletDiscountFactorsUp = floor->result<std::vector<Real> >("optionletsDiscountFactor");
+    capletForwardsUp = cap->result<std::vector<Real> >("optionletsAtmForward");
+    floorletForwardsUp = floor->result<std::vector<Real> >("optionletsAtmForward");
+    
+    spread->setValue(-eps);
+    capletDownPrices = cap->result<std::vector<Real> >("optionletsPrice");
+    floorletDownPrices = floor->result<std::vector<Real> >("optionletsPrice");
+    capletDiscountFactorsDown = cap->result<std::vector<Real> >("optionletsDiscountFactor");
+    floorletDiscountFactorsDown = floor->result<std::vector<Real> >("optionletsDiscountFactor");
+    capletForwardsDown = cap->result<std::vector<Real> >("optionletsAtmForward");
+    floorletForwardsDown = floor->result<std::vector<Real> >("optionletsAtmForward");
+
+    Real accrualFactor;
+    Leg capLeg = cap->floatingLeg();
+    Leg floorLeg = floor->floatingLeg();
+    
+    for (Size n=1; n < capletUpPrices.size(); n++){
+        // calculating only caplet's FD sensitivity w.r.t. forward rate
+        // without the effect of sensitivity related to changed discount factor
+        ext::shared_ptr<FloatingRateCoupon> c = ext::dynamic_pointer_cast<FloatingRateCoupon>(capLeg[n]);
+        accrualFactor = c->nominal() * c->accrualPeriod() * c->gearing();
+        capletFDDelta[n] = (capletUpPrices[n] / capletDiscountFactorsUp[n]
+                           - capletDownPrices[n] / capletDiscountFactorsDown[n]) 
+                           / (capletForwardsUp[n] - capletForwardsDown[n])
+                           / accrualFactor;
+    }
+
+    for (Size n=0; n<floorletUpPrices.size(); n++){
+        // calculating only caplet's FD sensitivity w.r.t. forward rate
+        // without the effect of sensitivity related to changed discount factor
+        ext::shared_ptr<FloatingRateCoupon> c = ext::dynamic_pointer_cast<FloatingRateCoupon>(floorLeg[n]);
+        accrualFactor = c->nominal() * c->accrualPeriod() * c->gearing();
+        floorletFDDelta[n] = (floorletUpPrices[n] / floorletDiscountFactorsUp[n] 
+                             - floorletDownPrices[n] / floorletDiscountFactorsDown[n]) 
+                             / (floorletForwardsUp[n] - floorletForwardsDown[n])
+                             / accrualFactor;        
+    }
+
+    for (Size n=0; n<capletAnalyticDelta.size(); n++){
+        if (std::fabs(capletAnalyticDelta[n]-capletFDDelta[n]) > 1.0e-6)
+            BOOST_ERROR(
+                "failed to compare analytical and finite difference caplet delta:\n"
+                << "caplet number:\t" << n << "\n"
+                << std::setprecision(12)
+                << "    finite difference: " << capletFDDelta[n]<< "\n"
+                << "    analytical value:   " << capletAnalyticDelta[n] << "\n"
+                << "    resulting ratio: " << capletFDDelta[n] / capletAnalyticDelta[n]);    
+    }
+
+    for (Size n=0; n<floorletAnalyticDelta.size(); n++){
+        if (std::fabs(floorletAnalyticDelta[n]-floorletFDDelta[n]) > 1.0e-6)
+            BOOST_ERROR(
+                "failed to compare analytical and finite difference floorlet delta:\n"
+                << "floorlet number:\t" << n << "\n"
+                << std::setprecision(12)
+                << "    finite difference: " << floorletFDDelta[n]<< "\n"
+                << "    analytical value:   " << floorletAnalyticDelta[n] << "\n"
+                << "    resulting ratio: " << floorletFDDelta[n] / floorletAnalyticDelta[n]);    
+    }
+
+}
+
+void CapFloorTest::testBachelierOptionLetsDelta() {
+
+    BOOST_TEST_MESSAGE("Testing Bachelier caplet/floorlet delta coefficients against finite difference values...");
+
+    using namespace capfloor_test;
+
+    CommonVars vars;
+
+    Date cachedToday(14,March,2002),
+         cachedSettlement(18,March,2002);
+    Settings::instance().evaluationDate() = cachedToday;
+    ext::shared_ptr<YieldTermStructure> baseCurve = flatRate(cachedSettlement, 
+                                                             0.05, Actual360());
+    RelinkableHandle<YieldTermStructure> baseCurveHandle(baseCurve);
+
+    // Define spreaded curve with eps as spread used for FD sensitivities
+    Real eps = 1.0e-6;
+    ext::shared_ptr<SimpleQuote> spread(new SimpleQuote(0.0));
+    ext::shared_ptr<YieldTermStructure> spreadCurve(new ZeroSpreadedTermStructure(
+                                                            baseCurveHandle,
+                                                            Handle<Quote>(spread),
+                                                            Continuous,
+                                                            Annual,
+                                                            Actual360()));                                               
+    vars.termStructure.linkTo(spreadCurve);
+    Date startDate = vars.termStructure->referenceDate();
+    Leg leg = vars.makeLeg(startDate,20);  
+
+    // Use normal model (BachelierCapFloorEngine)
+    bool isLogNormal = false;
+
+    ext::shared_ptr<CapFloor> cap = vars.makeCapFloor(CapFloor::Cap,leg,
+                                                      0.05, 0.01, isLogNormal);
+    ext::shared_ptr<CapFloor> floor = vars.makeCapFloor(CapFloor::Floor,leg,
+                                                        0.05, 0.01, isLogNormal);
+
+    
+    //so far tests pass, now try to get additional results and it will fail
+    Size capletsNum = cap->capRates().size();
+    std::vector<Real> capletUpPrices, 
+                      capletDownPrices,
+                      capletAnalyticDelta,
+                      capletDiscountFactorsUp,
+                      capletDiscountFactorsDown,
+                      capletForwardsUp,
+                      capletForwardsDown,
+                      capletFDDelta(capletsNum, 0.0); 
+    Size floorletNum = floor->floorRates().size();
+    std::vector<Real> floorletUpPrices, 
+                      floorletDownPrices,
+                      floorletAnalyticDelta,
+                      floorletDiscountFactorsUp,
+                      floorletDiscountFactorsDown,
+                      floorletForwardsUp,
+                      floorletForwardsDown,
+                      floorletFDDelta(floorletNum, 0.0);
+    
+    capletAnalyticDelta = cap->result<std::vector<Real> >("optionletsDelta");
+    floorletAnalyticDelta = floor->result<std::vector<Real> >("optionletsDelta");
+    
+    spread->setValue(eps);
+    capletUpPrices = cap->result<std::vector<Real> >("optionletsPrice");
+    floorletUpPrices = floor->result<std::vector<Real> >("optionletsPrice");
+    capletDiscountFactorsUp = cap->result<std::vector<Real> >("optionletsDiscountFactor");
+    floorletDiscountFactorsUp = floor->result<std::vector<Real> >("optionletsDiscountFactor");
+    capletForwardsUp = cap->result<std::vector<Real> >("optionletsAtmForward");
+    floorletForwardsUp = floor->result<std::vector<Real> >("optionletsAtmForward");
+    
+    spread->setValue(-eps);
+    capletDownPrices = cap->result<std::vector<Real> >("optionletsPrice");
+    floorletDownPrices = floor->result<std::vector<Real> >("optionletsPrice");
+    capletDiscountFactorsDown = cap->result<std::vector<Real> >("optionletsDiscountFactor");
+    floorletDiscountFactorsDown = floor->result<std::vector<Real> >("optionletsDiscountFactor");
+    capletForwardsDown = cap->result<std::vector<Real> >("optionletsAtmForward");
+    floorletForwardsDown = floor->result<std::vector<Real> >("optionletsAtmForward");
+
+    Real accrualFactor;
+    Leg capLeg = cap->floatingLeg();
+    Leg floorLeg = floor->floatingLeg();
+    
+    for (Size n=1; n < capletUpPrices.size(); n++){
+        // calculating only caplet's FD sensitivity w.r.t. forward rate
+        // without the effect of sensitivity related to changed discount factor
+        ext::shared_ptr<FloatingRateCoupon> c = ext::dynamic_pointer_cast<FloatingRateCoupon>(capLeg[n]);
+        accrualFactor = c->nominal() * c->accrualPeriod() * c->gearing();
+        capletFDDelta[n] = (capletUpPrices[n] / capletDiscountFactorsUp[n]
+                           - capletDownPrices[n] / capletDiscountFactorsDown[n]) 
+                           / (capletForwardsUp[n] - capletForwardsDown[n])
+                           / accrualFactor;
+    }
+
+    for (Size n=0; n<floorletUpPrices.size(); n++){
+        // calculating only caplet's FD sensitivity w.r.t. forward rate
+        // without the effect of sensitivity related to changed discount factor
+        ext::shared_ptr<FloatingRateCoupon> c = ext::dynamic_pointer_cast<FloatingRateCoupon>(floorLeg[n]);
+        accrualFactor = c->nominal() * c->accrualPeriod() * c->gearing();
+        floorletFDDelta[n] = (floorletUpPrices[n] / floorletDiscountFactorsUp[n] 
+                             - floorletDownPrices[n] / floorletDiscountFactorsDown[n]) 
+                             / (floorletForwardsUp[n] - floorletForwardsDown[n])
+                             / accrualFactor;        
+    }
+
+    for (Size n=0; n<capletAnalyticDelta.size(); n++){
+        if (std::fabs(capletAnalyticDelta[n]-capletFDDelta[n]) > 1.0e-6)
+            BOOST_ERROR(
+                "failed to compare analytical and finite difference caplet delta:\n"
+                << "caplet number:\t" << n << "\n"
+                << std::setprecision(12)
+                << "    finite difference: " << capletFDDelta[n]<< "\n"
+                << "    analytical value:   " << capletAnalyticDelta[n] << "\n"
+                << "    resulting ratio: " << capletFDDelta[n] / capletAnalyticDelta[n]);    
+    }
+
+    for (Size n=0; n<floorletAnalyticDelta.size(); n++){
+        if (std::fabs(floorletAnalyticDelta[n]-floorletFDDelta[n]) > 1.0e-6)
+            BOOST_ERROR(
+                "failed to compare analytical and finite difference floorlet delta:\n"
+                << "floorlet number:\t" << n << "\n"
+                << std::setprecision(12)
+                << "    finite difference: " << floorletFDDelta[n]<< "\n"
+                << "    analytical value:   " << floorletAnalyticDelta[n] << "\n"
+                << "    resulting ratio: " << floorletFDDelta[n] / floorletAnalyticDelta[n]);    
+    }
+
+}
 
 test_suite* CapFloorTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("Cap and floor tests");
@@ -621,6 +970,9 @@ test_suite* CapFloorTest::suite() {
     suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testATMRate));
     suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testImpliedVolatility));
     suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testCachedValue));
+    suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testCachedValueFromOptionLets));
+    suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testOptionLetsDelta));
+    suite->add(QUANTLIB_TEST_CASE(&CapFloorTest::testBachelierOptionLetsDelta));
     return suite;
 }
 
