@@ -50,14 +50,7 @@
 #include <ql/pricingengines/vanilla/analytichestonhullwhiteengine.hpp>
 #include <ql/pricingengines/vanilla/fdhestonvanillaengine.hpp>
 #include <ql/pricingengines/vanilla/fdhestonhullwhitevanillaengine.hpp>
-#if defined(__GNUC__) && (((__GNUC__ == 4) && (__GNUC_MINOR__ >= 8)) || (__GNUC__ > 4))
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-local-typedefs"
-#endif
-#include <boost/bind.hpp>
-#if defined(__GNUC__) && (((__GNUC__ == 4) && (__GNUC_MINOR__ >= 8)) || (__GNUC__ > 4))
-#pragma GCC diagnostic pop
-#endif
+#include <ql/functional.hpp>
 
 using namespace QuantLib;
 using namespace boost::unit_test_framework;
@@ -84,7 +77,6 @@ void HybridHestonHullWhiteProcessTest::testBsmHullWhiteEngine() {
     ext::shared_ptr<SimpleQuote> vol(new SimpleQuote(0.25));
     const Handle<BlackVolTermStructure> volTS(flatVol(today, vol, dc));
 
-    // FLOATING_POINT_EXCEPTION
     ext::shared_ptr<HullWhite> hullWhiteModel(
         new HullWhite(Handle<YieldTermStructure>(rTS), 0.00883, 0.00526));
 
@@ -652,6 +644,8 @@ void HybridHestonHullWhiteProcessTest::testAnalyticHestonHullWhitePricing() {
 void HybridHestonHullWhiteProcessTest::testCallableEquityPricing() {
     BOOST_TEST_MESSAGE("Testing the pricing of a callable equity product...");
 
+    using namespace ext::placeholders;
+
     SavedSettings backup;
 
     /*
@@ -693,7 +687,7 @@ void HybridHestonHullWhiteProcessTest::testCallableEquityPricing() {
 
     std::vector<Time> times(maturity+1);
     std::transform(schedule.begin(), schedule.end(), times.begin(),
-                   boost::bind(&Actual365Fixed::yearFraction,
+                   ext::bind(&Actual365Fixed::yearFraction,
                                dc, today, _1, Date(), Date()));
 
     for (Size i=0; i<=maturity; ++i)
@@ -867,7 +861,7 @@ void HybridHestonHullWhiteProcessTest::testFdmHestonHullWhiteEngine() {
     Settings::instance().evaluationDate() = today;
     const Date exerciseDate = Date(28, March, 2012);
     DayCounter dc = Actual365Fixed();
-    
+
     Handle<Quote> s0(ext::shared_ptr<Quote>(new SimpleQuote(100.0)));
 
     const Handle<YieldTermStructure> rTS(flatRate(0.05, dc));
@@ -887,7 +881,7 @@ void HybridHestonHullWhiteProcessTest::testFdmHestonHullWhiteEngine() {
                               new HullWhiteProcess(rTS, 0.00883, 0.01));
     ext::shared_ptr<HullWhite> hwModel(
                     new HullWhite(rTS, hwProcess->a(), hwProcess->sigma()));
-    
+
     ext::shared_ptr<Exercise> exercise(new EuropeanExercise(exerciseDate));
     const Real corr[] = {-0.85, 0.5 };
     const Real strike[] = { 75, 120, 160 };
@@ -897,7 +891,7 @@ void HybridHestonHullWhiteProcessTest::testFdmHestonHullWhiteEngine() {
             ext::shared_ptr<StrikedTypePayoff> payoff(
                               new PlainVanillaPayoff(Option::Call, strike[j]));
             VanillaOption option(payoff, exercise);
-            
+
             option.setPricingEngine(ext::shared_ptr<PricingEngine>(
                 new FdHestonHullWhiteVanillaEngine(
                     ext::make_shared<HestonModel>(
@@ -906,7 +900,7 @@ void HybridHestonHullWhiteProcessTest::testFdmHestonHullWhiteEngine() {
             const Real calculated = option.NPV();
             const Real calculatedDelta = option.delta();
             const Real calculatedGamma = option.gamma();
-            
+
             option.setPricingEngine(ext::shared_ptr<PricingEngine>(
                 new AnalyticBSMHullWhiteEngine(corr[i],
                                                stochProcess, hwModel)));
@@ -943,7 +937,7 @@ void HybridHestonHullWhiteProcessTest::testFdmHestonHullWhiteEngine() {
 }
 
 
-namespace {
+namespace hybrid_heston_hullwhite_process_test {
 
     struct HestonModelData {
         const char* const name;
@@ -955,18 +949,18 @@ namespace {
         Real r;
         Real q;
     };
-    
+
     HestonModelData hestonModels[] = {
-        // ADI finite difference schemes for option pricing in the 
+        // ADI finite difference schemes for option pricing in the
         // Heston model with correlation, K.J. in t'Hout and S. Foulon,
         {"'t Hout case 1", 0.04, 1.5, 0.04, 0.3, -0.9, 0.025, 0.0},
         {"'t Hout case 2", 0.12, 3.0, 0.12, 0.04, 0.6, 0.01, 0.04},
         {"'t Hout case 3", 0.0707,0.6067, 0.0707, 0.2928, -0.7571, 0.03, 0.0},
         {"'t Hout case 4", 0.06, 2.5, 0.06, 0.5, -0.1, 0.0507, 0.0469},
-        // Efficient numerical methods for pricing American options under 
+        // Efficient numerical methods for pricing American options under
         // stochastic volatility, Samuli Ikonen and Jari Toivanen,
         {"Ikonen-Toivanen", 0.0625, 5, 0.16, 0.9, 0.1, 0.1, 0.0},
-        // Not-so-complex logarithms in the Heston model, 
+        // Not-so-complex logarithms in the Heston model,
         // Christian Kahl and Peter Jäckel
         {"Kahl-Jaeckel", 0.16, 1.0, 0.16, 2.0, -0.8, 0.0, 0.0},
         // self defined test cases
@@ -981,17 +975,17 @@ namespace {
         Real a;
         Real sigma;
     };
-    
+
     HullWhiteModelData hullWhiteModels[] = {
         {"EUR-2003", 0.00883, 0.00631 }
     };
-    
-    
+
+
     struct SchemeData {
         const char* const name;
         FdmSchemeDesc schemeDesc;
     };
-        
+
     SchemeData schemes[] = {
         { "HV2", FdmSchemeDesc::Hundsdorfer() },
         { "HV1", FdmSchemeDesc::ModifiedHundsdorfer() },
@@ -1015,21 +1009,21 @@ namespace {
         DayCounter dayCounter = Actual365Fixed();
         Handle<YieldTermStructure> rTS(flatRate(params.r, dayCounter));
         Handle<YieldTermStructure> qTS(flatRate(params.q, dayCounter));
-        
+
         return ext::make_shared<HestonProcess>(
-                   rTS, qTS, spot, params.v0, params.kappa, 
+                   rTS, qTS, spot, params.v0, params.kappa,
                    params.theta, params.sigma, params.rho);
     }
-    
+
     ext::shared_ptr<VanillaOption> makeVanillaOption(
                                             const VanillaOptionData& params) {
-        
-        Date maturity = Date(Settings::instance().evaluationDate()) 
+
+        Date maturity = Date(Settings::instance().evaluationDate())
                                           + Period(Size(params.maturity*365), Days);
         ext::shared_ptr<Exercise> exercise(new EuropeanExercise(maturity));
         ext::shared_ptr<StrikedTypePayoff> payoff(
                     new PlainVanillaPayoff(params.optionType, params.strike));
-        
+
         return ext::make_shared<VanillaOption>(
                                           payoff, exercise);
     }
@@ -1037,9 +1031,11 @@ namespace {
 
 void HybridHestonHullWhiteProcessTest::testBsmHullWhitePricing() {
     BOOST_TEST_MESSAGE("Testing convergence speed of Heston-Hull-White engine...");
-    
+
+    using namespace hybrid_heston_hullwhite_process_test;
+
     SavedSettings backup;
-    
+
     Date today(27, December, 2004);
     Settings::instance().evaluationDate() = today;
 
@@ -1048,33 +1044,33 @@ void HybridHestonHullWhiteProcessTest::testBsmHullWhitePricing() {
     Real strikes[] = {75,85,90,95,100,105,110,115,120,125,130,140,150};
     Size listOfTimeStepsPerYear[] = { 20 };
 
-    HestonModelData hestonModelData 
+    HestonModelData hestonModelData
         = { "BSM-HW Model", 0.09, 1.0, 0.09, QL_EPSILON, 0.0, 0.04, 0.03 };
     HullWhiteModelData hwModelData = hullWhiteModels[0];
     bool controlVariate[] = { true, false };
 
     ext::shared_ptr<HestonProcess> hp(makeHestonProcess(hestonModelData));
     ext::shared_ptr<HestonModel> hestonModel(new HestonModel(hp));
-    
+
     ext::shared_ptr<HullWhiteProcess> hwProcess(
         new HullWhiteProcess(hp->riskFreeRate(),
                              hwModelData.a, hwModelData.sigma));
     ext::shared_ptr<HullWhite> hullWhiteModel(
-        new HullWhite(hp->riskFreeRate(), 
+        new HullWhite(hp->riskFreeRate(),
                       hwProcess->a(), hwProcess->sigma()));
 
 
     ext::shared_ptr<BlackScholesMertonProcess> bsmProcess(
         new BlackScholesMertonProcess(
-            hp->s0(), hp->dividendYield(), hp->riskFreeRate(), 
+            hp->s0(), hp->dividendYield(), hp->riskFreeRate(),
             Handle<BlackVolTermStructure>(
                 flatVol(today, std::sqrt(hestonModelData.theta),
                         hp->riskFreeRate()->dayCounter()))));
-    
+
     ext::shared_ptr<PricingEngine> bsmhwEngine(
                      new AnalyticBSMHullWhiteEngine(equityIrCorr, bsmProcess,
                                                     hullWhiteModel));
-    
+
     Real tolWithCV[]    = { 2e-4, 2e-4, 2e-4, 2e-4, 0.01 };
     Real tolWithOutCV[] = { 5e-3, 5e-3, 5e-3, 5e-3, 0.02 };
     for (Size l=0; l < LENGTH(schemes); ++l) {
@@ -1082,61 +1078,63 @@ void HybridHestonHullWhiteProcessTest::testBsmHullWhitePricing() {
         for (Size i=0; i < LENGTH(controlVariate); ++i) {
             for (Size u=0; u < LENGTH(listOfTimeStepsPerYear); ++u) {
                 Size tSteps = Size(maturity*listOfTimeStepsPerYear[u]);
-    
+
                 ext::shared_ptr<FdHestonHullWhiteVanillaEngine> fdEngine(
                     new FdHestonHullWhiteVanillaEngine(
-                                    hestonModel, hwProcess, equityIrCorr,  
+                                    hestonModel, hwProcess, equityIrCorr,
                                     tSteps, 400, 2, 10, 0, controlVariate[i],
                                     scheme.schemeDesc));
                 fdEngine->enableMultipleStrikesCaching(
                     std::vector<Real>(strikes, strikes + LENGTH(strikes)));
-    
+
                 Real avgPriceDiff = 0.0;
                 for (Size k=0; k < LENGTH(strikes); ++k) {
-                    VanillaOptionData optionData 
-                                  = { strikes[k], maturity, Option::Call }; 
-                    ext::shared_ptr<VanillaOption> option 
+                    VanillaOptionData optionData
+                                  = { strikes[k], maturity, Option::Call };
+                    ext::shared_ptr<VanillaOption> option
                                         = makeVanillaOption(optionData);
                     option->setPricingEngine(bsmhwEngine);
                     Real expected = option->NPV();
-     
+
                     option->setPricingEngine(fdEngine);
                     Real calculated = option->NPV();
                     avgPriceDiff
                         +=std::fabs(expected-calculated)/LENGTH(strikes);
                 }
-                
+
                 if (controlVariate[i] && tolWithCV[l] < avgPriceDiff) {
                      BOOST_ERROR("Failed to reproduce BSM-Hull-White prices"
                              << "\n   scheme       : " << scheme.name
                              << "\n   model        : " << hestonModelData.name
-                             << "\n   CV           : on");                     
+                             << "\n   CV           : on");
                 }
-                
-                
+
+
                 if (!controlVariate[i] && tolWithOutCV[l] < avgPriceDiff) {
                     BOOST_ERROR("Failed to reproduce BSM-Hull-White prices"
                             << "\n   scheme       : " << scheme.name
                             << "\n   model        : " << hestonModelData.name
-                            << "\n   CV           : off");                     
+                            << "\n   CV           : off");
                 }
             }
-        }    
+        }
     }
 }
 
 void HybridHestonHullWhiteProcessTest::testSpatialDiscretizatinError() {
     BOOST_TEST_MESSAGE("Testing spatial convergence speed of Heston engine...");
-    
+
+    using namespace hybrid_heston_hullwhite_process_test;
+
     SavedSettings backup;
-    
+
     Date today(27, December, 2004);
     Settings::instance().evaluationDate() = today;
 
     Real maturity=1.0;
     Real strikes[] = {75,85,90,95,100,105,110,115,120,125,130,140,150};
     Size listOfTimeStepsPerYear[] = { 40 };
-    
+
     const Real tol[] = { 0.02, 0.02, 0.02, 0.02, 0.05 };
     for (Size u=0; u < LENGTH(listOfTimeStepsPerYear); ++u) {
         for (Size i=0; i < LENGTH(schemes); ++i) {
@@ -1146,34 +1144,34 @@ void HybridHestonHullWhiteProcessTest::testSpatialDiscretizatinError() {
                                         makeHestonProcess(hestonModels[j]));
                 ext::shared_ptr<HestonModel> hestonModel(
                                         new HestonModel(hestonProcess));
-    
+
                 ext::shared_ptr<PricingEngine> analyticEngine(
                                new AnalyticHestonEngine(hestonModel, 172));
-    
+
                 Size tSteps = Size(maturity*listOfTimeStepsPerYear[u]);
-    
+
                 ext::shared_ptr<FdHestonVanillaEngine> fdEngine(
                     new FdHestonVanillaEngine(
                         hestonModel, tSteps, 200, 40, 0,
                         schemes[i].schemeDesc));
                 fdEngine->enableMultipleStrikesCaching(
                     std::vector<Real>(strikes, strikes + LENGTH(strikes)));
-    
+
                 for (Size k=0; k < LENGTH(strikes); ++k) {
-                    VanillaOptionData optionData 
-                                  = { strikes[k], maturity, Option::Call }; 
-                    ext::shared_ptr<VanillaOption> option 
+                    VanillaOptionData optionData
+                                  = { strikes[k], maturity, Option::Call };
+                    ext::shared_ptr<VanillaOption> option
                                         = makeVanillaOption(optionData);
                     option->setPricingEngine(analyticEngine);
                     Real expected = option->NPV();
-     
+
                     option->setPricingEngine(fdEngine);
                     Real calculated = option->NPV();
-     
+
                     avgPriceDiff
                         +=std::fabs(expected-calculated)/LENGTH(strikes);
                 }
-    
+
                 if (avgPriceDiff > tol[i]) {
                     BOOST_ERROR("\nFailed to reproduce Heston prices" <<
                         "\n scheme    : " << schemes[i].name <<
@@ -1189,7 +1187,7 @@ void HybridHestonHullWhiteProcessTest::testSpatialDiscretizatinError() {
 
 
 
-namespace {
+namespace hybrid_heston_hullwhite_process_test {
     class HestonHullWhiteCorrelationConstraint : public Constraint {
       private:
         class Impl : public Constraint::Impl {
@@ -1219,15 +1217,17 @@ namespace {
 void HybridHestonHullWhiteProcessTest::testHestonHullWhiteCalibration() {
     BOOST_TEST_MESSAGE("Testing the Heston Hull-White calibration...");
 
-    // Calibration of a hybrid Heston-Hull-White model using 
+    using namespace hybrid_heston_hullwhite_process_test;
+
+    // Calibration of a hybrid Heston-Hull-White model using
     // the finite difference HestonHullWhite pricing engine
     //
     // Input surface is based on a Heston-Hull-White model with
     // Hull-White: a = 0.00883, \sigma = 0.00631
-    // Heston    : \nu = 0.12, \kappa = 2.0, 
+    // Heston    : \nu = 0.12, \kappa = 2.0,
     //             \theta = 0.09, \sigma = 0.5, \rho=-0.75
     // Equity Short rate correlation: -0.5
-    
+
     SavedSettings backup;
 
     const DayCounter dc = Actual365Fixed();
@@ -1237,13 +1237,13 @@ void HybridHestonHullWhiteProcessTest::testHestonHullWhiteCalibration() {
 
     const Handle<YieldTermStructure> rTS(flatRate(0.05, dc));
 
-    // assuming, that the Hull-White process is already calibrated 
+    // assuming, that the Hull-White process is already calibrated
     // on a given set of pure interest rate calibration instruments.
     ext::shared_ptr<HullWhiteProcess> hwProcess(
                               new HullWhiteProcess(rTS, 0.00883, 0.00631));
     ext::shared_ptr<HullWhite> hullWhiteModel(
                     new HullWhite(rTS, hwProcess->a(), hwProcess->sigma()));
-                    
+
     const Handle<YieldTermStructure> qTS(flatRate(0.02, dc));
     Handle<Quote> s0(ext::shared_ptr<Quote>(new SimpleQuote(100.0)));
 
@@ -1253,9 +1253,9 @@ void HybridHestonHullWhiteProcessTest::testHestonHullWhiteCalibration() {
     const Real start_kappa = 0.5;
     const Real start_sigma = 0.25;
     const Real start_rho   = -0.5;
-    
+
     const ext::shared_ptr<HestonProcess> hestonProcess(
-        new HestonProcess(rTS, qTS, s0, start_v0, start_kappa, 
+        new HestonProcess(rTS, qTS, s0, start_v0, start_kappa,
                           start_theta, start_sigma, start_rho));
     const ext::shared_ptr<HestonModel> analyticHestonModel
                                             (new HestonModel(hestonProcess));
@@ -1268,7 +1268,7 @@ void HybridHestonHullWhiteProcessTest::testHestonHullWhiteCalibration() {
 
     const Real strikes[]    = { 50, 75, 90, 100, 110, 125, 150, 200 };
     const Time maturities[] = { 1/12., 3/12., 0.5, 1.0, 2.0, 3.0, 5.0, 7.5, 10};
-    
+
     const Volatility vol[] = {
         0.482627,0.407617,0.366682,0.340110,0.314266,0.280241,0.252471,0.325552,
         0.464811,0.393336,0.354664,0.329758,0.305668,0.273563,0.244024,0.244886,
@@ -1280,9 +1280,9 @@ void HybridHestonHullWhiteProcessTest::testHestonHullWhiteCalibration() {
         0.311278,0.296340,0.289481,0.285482,0.281840,0.276924,0.269856,0.258609,
         0.303219,0.291534,0.286187,0.283073,0.280239,0.276414,0.270926,0.262173
     };
-    
-    std::vector<ext::shared_ptr<BlackCalibrationHelper> > options;
-    
+
+    std::vector<ext::shared_ptr<CalibrationHelper> > options;
+
     for (Size i=0; i < LENGTH(maturities); ++i) {
         const Period maturity((int)(maturities[i]*12.0+0.5), Months);
         ext::shared_ptr<Exercise> exercise(
@@ -1297,19 +1297,21 @@ void HybridHestonHullWhiteProcessTest::testHestonHullWhiteCalibration() {
                 strikes[j]));
             RelinkableHandle<Quote> v(ext::shared_ptr<Quote>(
                                    new SimpleQuote(vol[i*LENGTH(strikes)+j])));
-            options.push_back(ext::shared_ptr<BlackCalibrationHelper>(
+
+            ext::shared_ptr<BlackCalibrationHelper> helper(
                 new HestonModelHelper(maturity, calendar, s0,
                                       strikes[j], v, rTS, qTS,
-                                      BlackCalibrationHelper::PriceError)));
-            const Real marketValue = options.back()->marketValue();
-            
-            // Improve the quality of the starting point 
-            // for the full Heston-Hull-White calibration 
+                                      BlackCalibrationHelper::PriceError));
+            options.push_back(helper);
+            const Real marketValue = helper->marketValue();
+
+            // Improve the quality of the starting point
+            // for the full Heston-Hull-White calibration
             ext::shared_ptr<SimpleQuote> volQuote(new SimpleQuote);
             ext::shared_ptr<GeneralizedBlackScholesProcess> bsProcess =
-                detail::ImpliedVolatilityHelper::clone(
+                QuantLib::detail::ImpliedVolatilityHelper::clone(
                     ext::make_shared<GeneralizedBlackScholesProcess>(
-                        
+
                             s0, qTS, rTS, Handle<BlackVolTermStructure>(
                                                     flatVol(v->value(), dc))),
                         volQuote);
@@ -1320,17 +1322,17 @@ void HybridHestonHullWhiteProcessTest::testHestonHullWhiteCalibration() {
                 new AnalyticBSMHullWhiteEngine(equityShortRateCorr,
                                                bsProcess, hullWhiteModel));
 
-            Volatility vt = detail::ImpliedVolatilityHelper::calculate(
+            Volatility vt = QuantLib::detail::ImpliedVolatilityHelper::calculate(
                 dummyOption, *bshwEngine, *volQuote,
                 marketValue, 1e-8, 100, 0.0001, 10);
 
             v.linkTo(ext::shared_ptr<Quote>(new SimpleQuote(vt)));
-            
-            options.back()->setPricingEngine(
+
+            helper->setPricingEngine(
                 ext::shared_ptr<PricingEngine>(analyticHestonEngine));
         }
-    }    
-        
+    }
+
     HestonHullWhiteCorrelationConstraint corrConstraint(equityShortRateCorr);
     LevenbergMarquardt om(1e-6, 1e-8, 1e-8);
     analyticHestonModel->calibrate(options, om,
@@ -1343,68 +1345,69 @@ void HybridHestonHullWhiteProcessTest::testHestonHullWhiteCalibration() {
     for (Size i=0; i < LENGTH(maturities); ++i) {
         const Size tGrid = static_cast<Size>(std::max(5.0, maturities[i]*5.0));
         ext::shared_ptr<FdHestonHullWhiteVanillaEngine> engine(
-            new FdHestonHullWhiteVanillaEngine(fdmHestonModel, hwProcess, 
-                                               equityShortRateCorr, 
+            new FdHestonHullWhiteVanillaEngine(fdmHestonModel, hwProcess,
+                                               equityShortRateCorr,
                                                tGrid, 45, 11, 5, 0, true));
-        
+
         engine->enableMultipleStrikesCaching(
                      std::vector<Real>(strikes, strikes + LENGTH(strikes)));
-        
+
         const Period maturity((int)(maturities[i]*12.0+0.5), Months);
-        
+
         for (Size j=0; j < LENGTH(strikes); ++j) {
             // multiple strikes engine works best if the first option
             // per maturity has the average strike (because the first option
             // is priced first during the calibration and the first pricing
             // is used to calculate the prices for all strikes
             const Size js = (j + (LENGTH(strikes)-1)/2) % LENGTH(strikes);
- 
+
             ext::shared_ptr<StrikedTypePayoff> payoff(
                              new PlainVanillaPayoff(Option::Call, strikes[js]));
             Handle<Quote> v(ext::shared_ptr<Quote>(
                                    new SimpleQuote(vol[i*LENGTH(strikes)+js])));
-            options.push_back(ext::shared_ptr<BlackCalibrationHelper>(
+            ext::shared_ptr<BlackCalibrationHelper> helper(
                 new HestonModelHelper(maturity, calendar, s0,
                                       strikes[js], v, rTS, qTS,
-                                      BlackCalibrationHelper::PriceError)));
-            
-            options.back()->setPricingEngine(engine);
+                                      BlackCalibrationHelper::PriceError));
+            options.push_back(helper);
+
+            helper->setPricingEngine(engine);
         }
-    }    
+    }
 
     LevenbergMarquardt vm(1e-6, 1e-2, 1e-2);
-    fdmHestonModel->calibrate(options, vm, 
+    fdmHestonModel->calibrate(options, vm,
                               EndCriteria(400, 40, 1.0e-8, 1.0e-4, 1.0e-8),
                               corrConstraint);
-    
+
     const Real relTol = 0.01;
     const Real expected_v0    =  0.12;
     const Real expected_kappa =  2.0;
     const Real expected_theta =  0.09;
     const Real expected_sigma =  0.5;
     const Real expected_rho   = -0.75;
-    
+
     if (std::fabs(fdmHestonModel->v0() - expected_v0)/expected_v0 > relTol) {
          BOOST_ERROR("Failed to reproduce Heston-Hull-White model"
                  << "\n   v0 calculated: " << fdmHestonModel->v0()
                  << "\n   v0 expected  : " << expected_v0
                  << "\n   relatove tol : " << relTol);
     }
-    if (std::fabs(fdmHestonModel->theta() - expected_theta)/expected_theta 
+    if (std::fabs(fdmHestonModel->theta() - expected_theta)/expected_theta
                                                                     > relTol) {
          BOOST_ERROR("Failed to reproduce Heston-Hull-White model"
                  << "\n   theta calculated: " << fdmHestonModel->theta()
                  << "\n   theta expected  : " << expected_theta
                  << "\n   relatove tol    : " << relTol);
     }
-    if (std::fabs(fdmHestonModel->kappa() - expected_kappa)/expected_kappa 
+    if (std::fabs(fdmHestonModel->kappa() - expected_kappa)/expected_kappa
                                                                     > relTol) {
         BOOST_ERROR("Failed to reproduce Heston-Hull-White model"
                 << "\n   kappa calculated: " << fdmHestonModel->kappa()
                 << "\n   kappa expected  : " << expected_kappa
                 << "\n   relatove tol    : " << relTol);
     }
-    if (std::fabs(fdmHestonModel->sigma() - expected_sigma)/expected_sigma 
+    if (std::fabs(fdmHestonModel->sigma() - expected_sigma)/expected_sigma
                                                                     > relTol) {
        BOOST_ERROR("Failed to reproduce Heston-Hull-White model"
                << "\n   sigma calculated: " << fdmHestonModel->sigma()
@@ -1500,7 +1503,7 @@ void HybridHestonHullWhiteProcessTest::testH1HWPricingEngine() {
         }
     }
 }
-    
+
 test_suite* HybridHestonHullWhiteProcessTest::suite(SpeedLevel speed) {
     test_suite* suite = BOOST_TEST_SUITE("Hybrid Heston-HullWhite tests");
 
@@ -1539,4 +1542,3 @@ test_suite* HybridHestonHullWhiteProcessTest::suite(SpeedLevel speed) {
 
     return suite;
 }
-
