@@ -52,13 +52,14 @@ using std::fabs;
 using std::pow;
 using std::vector;
 
+#undef REPORT_FAILURE
 #define REPORT_FAILURE(d, res, periodName) \
     BOOST_ERROR("wrong " << periodName << " inflation period for Date (1 " \
         << d << "), Start Date ( " \
         << res.first << "), End Date (" \
         << res.second << ")"); \
 
-namespace {
+namespace inflation_test {
 
     struct Datum {
         Date date;
@@ -77,7 +78,8 @@ namespace {
             const ext::shared_ptr<I> &ii, const Period &observationLag,
             const Calendar &calendar,
             const BusinessDayConvention &bdc,
-            const DayCounter &dc) {
+            const DayCounter &dc,
+            const Handle<YieldTermStructure>& yTS) {
 
         std::vector<ext::shared_ptr<BootstrapHelper<T> > > instruments;
         for (Size i=0; i<N; i++) {
@@ -86,7 +88,7 @@ namespace {
                 new SimpleQuote(iiData[i].rate/100.0)));
             ext::shared_ptr<BootstrapHelper<T> > anInstrument(new U(
                 quote, observationLag, maturity,
-                calendar, bdc, dc, ii));
+                calendar, bdc, dc, ii, yTS));
             instruments.push_back(anInstrument);
         }
 
@@ -98,7 +100,7 @@ namespace {
 // zero inflation tests, index, termstructure, and swaps
 //===========================================================================================
 
-namespace {
+namespace inflation_test {
 
 void checkSeasonality(const Handle<ZeroInflationTermStructure>& hz, 
     const ext::shared_ptr<ZeroInflationIndex>& ii) {
@@ -305,6 +307,8 @@ void InflationTest::testZeroIndex() {
 void InflationTest::testZeroTermStructure() {
     BOOST_TEST_MESSAGE("Testing zero inflation term structure...");
 
+    using namespace inflation_test;
+
     SavedSettings backup;
     IndexHistoryCleaner cleaner;
 
@@ -365,7 +369,8 @@ void InflationTest::testZeroTermStructure() {
     makeHelpers<ZeroInflationTermStructure,ZeroCouponInflationSwapHelper,
                 ZeroInflationIndex>(zcData, LENGTH(zcData), ii,
                                     observationLag,
-                                    calendar, bdc, dc);
+                                    calendar, bdc, dc,
+                                    Handle<YieldTermStructure>(nominalTS));
 
     Rate baseZeroRate = zcData[0].rate/100.0;
     ext::shared_ptr<PiecewiseZeroInflationCurve<Linear> > pZITS(
@@ -500,7 +505,8 @@ void InflationTest::testZeroTermStructure() {
     makeHelpers<ZeroInflationTermStructure,ZeroCouponInflationSwapHelper,
     ZeroInflationIndex>(zcData, LENGTH(zcData), iiyes,
                         observationLagyes,
-                        calendar, bdc, dc);
+                        calendar, bdc, dc,
+                        Handle<YieldTermStructure>(nominalTS));
 
     ext::shared_ptr<PiecewiseZeroInflationCurve<Linear> > pZITSyes(
             new PiecewiseZeroInflationCurve<Linear>(
@@ -792,6 +798,8 @@ void InflationTest::testYYIndex() {
 void InflationTest::testYYTermStructure() {
     BOOST_TEST_MESSAGE("Testing year-on-year inflation term structure...");
 
+    using namespace inflation_test;
+
     SavedSettings backup;
     IndexHistoryCleaner cleaner;
 
@@ -854,8 +862,9 @@ void InflationTest::testYYTermStructure() {
     std::vector<ext::shared_ptr<BootstrapHelper<YoYInflationTermStructure> > > helpers =
     makeHelpers<YoYInflationTermStructure,YearOnYearInflationSwapHelper,
     YoYInflationIndex>(yyData, LENGTH(yyData), iir,
-                        observationLag,
-                        calendar, bdc, dc);
+                       observationLag,
+                       calendar, bdc, dc,
+                       Handle<YieldTermStructure>(nominalTS));
 
     Rate baseYYRate = yyData[0].rate/100.0;
     ext::shared_ptr<PiecewiseYoYInflationCurve<Linear> > pYYTS(
