@@ -53,7 +53,7 @@
 using namespace QuantLib;
 using namespace boost::unit_test_framework;
 
-namespace {
+namespace libor_market_model_test {
 
     ext::shared_ptr<IborIndex> makeIndex(std::vector<Date> dates,
                                            std::vector<Rate> rates) {
@@ -115,6 +115,8 @@ namespace {
 void LiborMarketModelTest::testSimpleCovarianceModels() {
     BOOST_TEST_MESSAGE("Testing simple covariance models...");
 
+    using namespace libor_market_model_test;
+
     SavedSettings backup;
 
     const Size size = 10;
@@ -192,14 +194,16 @@ void LiborMarketModelTest::testSimpleCovarianceModels() {
 void LiborMarketModelTest::testCapletPricing() {
     BOOST_TEST_MESSAGE("Testing caplet pricing...");
 
+    using namespace libor_market_model_test;
+
     SavedSettings backup;
 
     const Size size = 10;
-    #if defined(QL_USE_INDEXED_COUPON)
-    const Real tolerance = 1e-5;
-    #else
-    const Real tolerance = 1e-12;
-    #endif
+    Real tolerance;
+    if (!IborCoupon::usingAtParCoupons())
+        tolerance = 1e-5;
+    else
+        tolerance = 1e-12;
 
     ext::shared_ptr<IborIndex> index = makeIndex();
     ext::shared_ptr<LiborForwardModelProcess> process(
@@ -245,6 +249,8 @@ void LiborMarketModelTest::testCapletPricing() {
 void LiborMarketModelTest::testCalibration() {
     BOOST_TEST_MESSAGE("Testing calibration of a Libor forward model...");
 
+    using namespace libor_market_model_test;
+
     SavedSettings backup;
 
     const Size size = 14;
@@ -288,7 +294,7 @@ void LiborMarketModelTest::testCalibration() {
     DayCounter dayCounter=index->forwardingTermStructure()->dayCounter();
 
     // set-up calibration helper
-    std::vector<ext::shared_ptr<BlackCalibrationHelper> > calibrationHelper;
+    std::vector<ext::shared_ptr<CalibrationHelper> > calibrationHelpers;
 
     Size i;
     for (i=2; i < size; ++i) {
@@ -304,7 +310,7 @@ void LiborMarketModelTest::testCalibration() {
         caphelper->setPricingEngine(ext::shared_ptr<PricingEngine>(
                            new AnalyticCapFloorEngine(model, termStructure)));
 
-        calibrationHelper.push_back(caphelper);
+        calibrationHelpers.push_back(caphelper);
 
         if (i<= size/2) {
             // add a few swaptions to test swaption calibration as well
@@ -325,18 +331,18 @@ void LiborMarketModelTest::testCalibration() {
                      ext::shared_ptr<PricingEngine>(
                                  new LfmSwaptionEngine(model,termStructure)));
 
-                calibrationHelper.push_back(swaptionHelper);
+                calibrationHelpers.push_back(swaptionHelper);
             }
         }
     }
 
     LevenbergMarquardt om(1e-6, 1e-6, 1e-6);
-    model->calibrate(calibrationHelper, om, EndCriteria(2000, 100, 1e-6, 1e-6, 1e-6));
+    model->calibrate(calibrationHelpers, om, EndCriteria(2000, 100, 1e-6, 1e-6, 1e-6));
 
     // measure the calibration error
     Real calculated = 0.0;
-    for (i=0; i<calibrationHelper.size(); ++i) {
-        Real diff = calibrationHelper[i]->calibrationError();
+    for (i=0; i<calibrationHelpers.size(); ++i) {
+        Real diff = calibrationHelpers[i]->calibrationError();
         calculated += diff*diff;
     }
 
@@ -349,15 +355,18 @@ void LiborMarketModelTest::testCalibration() {
 void LiborMarketModelTest::testSwaptionPricing() {
     BOOST_TEST_MESSAGE("Testing forward swap and swaption pricing...");
 
+    using namespace libor_market_model_test;
+
     SavedSettings backup;
 
     const Size size  = 10;
     const Size steps = 8*size;
-    #if defined(QL_USE_INDEXED_COUPON)
-    const Real tolerance = 1e-6;
-    #else
-    const Real tolerance = 1e-12;
-    #endif
+
+    Real tolerance;
+    if (!IborCoupon::usingAtParCoupons())
+        tolerance = 1e-6;
+    else
+        tolerance = 1e-12;
 
     std::vector<Date> dates;
     std::vector<Rate> rates;
@@ -412,9 +421,6 @@ void LiborMarketModelTest::testSwaptionPricing() {
 
     Date settlement  = index->forwardingTermStructure()->referenceDate();
 
-    ext::shared_ptr<SwaptionVolatilityMatrix> m =
-                liborModel->getSwaptionVolatilityMatrix();
-
     for (i=1; i < size; ++i) {
         for (Size j=1; j <= size-i; ++j) {
             Date fwdStart    = settlement + Period(6*i, Months);
