@@ -48,17 +48,8 @@
 #include <ql/experimental/processes/hestonslvprocess.hpp>
 #include <ql/experimental/finitedifferences/fdhestondoublebarrierengine.hpp>
 #include <ql/experimental/barrieroption/analyticdoublebarrierbinaryengine.hpp>
-
 #include <ql/experimental/volatility/sabrvoltermstructure.hpp>
-
-#if defined(__GNUC__) && (((__GNUC__ == 4) && (__GNUC_MINOR__ >= 8)) || (__GNUC__ > 4))
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wunused-local-typedefs"
-#endif
-#include <boost/bind.hpp>
-#if defined(__GNUC__) && (((__GNUC__ == 4) && (__GNUC_MINOR__ >= 8)) || (__GNUC__ > 4))
-#pragma GCC diagnostic pop
-#endif
+#include <ql/functional.hpp>
 
 #include <boost/assign/std/vector.hpp>
 
@@ -78,11 +69,11 @@ using namespace boost::assign;
 using namespace boost::unit_test_framework;
 
 
-namespace {
+namespace square_root_clv_model {
     class CLVModelPayoff : public PlainVanillaPayoff {
       public:
         CLVModelPayoff(Option::Type type, Real strike,
-                             const boost::function<Real(Real)> g)
+                             const ext::function<Real(Real)> g)
         : PlainVanillaPayoff(type, strike),
           g_(g) { }
 
@@ -91,8 +82,23 @@ namespace {
         }
 
       private:
-        const boost::function<Real(Real)> g_;
+        const ext::function<Real(Real)> g_;
     };
+
+    typedef boost::math::non_central_chi_squared_distribution<Real>
+        chi_squared_type;
+
+    class integrand {
+        CLVModelPayoff payoff;
+        chi_squared_type dist;
+      public:
+        integrand(const CLVModelPayoff& payoff, const chi_squared_type& dist)
+        : payoff(payoff), dist(dist) {}
+        Real operator()(Real x) const {
+            return payoff(x) * boost::math::pdf(dist, x);
+        }
+    };
+
 }
 
 
@@ -100,6 +106,9 @@ void SquareRootCLVModelTest::testSquareRootCLVVanillaPricing() {
     BOOST_TEST_MESSAGE(
         "Testing vanilla option pricing with square root kernel process...");
 
+    using namespace ext::placeholders;
+    using namespace square_root_clv_model;
+
     SavedSettings backup;
 
     const Date todaysDate(5, Oct, 2016);
@@ -147,11 +156,8 @@ void SquareRootCLVModelTest::testSquareRootCLVVanillaPricing() {
     const Real ncp = 4*kappa*std::exp(-kappa*maturity)
             / (sigma*sigma*(1-std::exp(-kappa*maturity)))*sqrtProcess->x0();
 
-    typedef boost::math::non_central_chi_squared_distribution<Real>
-        chi_squared_type;
-
     const chi_squared_type dist(df, ncp);
-
+        
     const Real strikes[] = { 50, 75, 100, 125, 150, 200 };
     for (Size i=0; i < LENGTH(strikes); ++i) {
         const Real strike = strikes[i];
@@ -165,11 +171,7 @@ void SquareRootCLVModelTest::testSquareRootCLVVanillaPricing() {
 
         const CLVModelPayoff clvModelPayoff(optionType, strike, g);
 
-        const boost::function<Real(Real)> f =
-            boost::bind(std::multiplies<Real>(),
-                boost::bind(&CLVModelPayoff::operator(), &clvModelPayoff, _1),
-                boost::bind<Real>(boost::math::pdf<chi_squared_type, Real>,
-                    dist, _1) );
+        const ext::function<Real(Real)> f = integrand(clvModelPayoff, dist);
 
         const Real calculated = GaussLobattoIntegral(1000, 1e-6)(
             f, x.front(), x.back()) * rTS->discount(maturity);
@@ -189,6 +191,8 @@ void SquareRootCLVModelTest::testSquareRootCLVMappingFunction() {
     BOOST_TEST_MESSAGE(
         "Testing mapping function of the square root kernel process...");
 
+    using namespace square_root_clv_model;
+
     SavedSettings backup;
 
     const Date todaysDate(16, Oct, 2016);
@@ -237,7 +241,7 @@ void SquareRootCLVModelTest::testSquareRootCLVMappingFunction() {
     const SquareRootCLVModel model(
         bsProcess, sqrtProcess, calibrationDates, 18, 1-1e-14, 1e-14);
 
-    const boost::function<Real(Time, Real)> g = model.g();
+    const ext::function<Real(Time, Real)> g = model.g();
 
     const Real strikes[] = { 80, 100, 120 };
     const Size offsets[] = { 7, 14, 28, 91, 182, 183, 184, 185, 186, 365 };
@@ -249,9 +253,6 @@ void SquareRootCLVModelTest::testSquareRootCLVMappingFunction() {
         const Real ncp = 4*kappa*std::exp(-kappa*t)
                 / (sigma*sigma*(1-std::exp(-kappa*t)))*sqrtProcess->x0();
 
-        typedef boost::math::non_central_chi_squared_distribution<Real>
-            chi_squared_type;
-
         const chi_squared_type dist(df, ncp);
 
         const Real fwd = s0*qTS->discount(m)/rTS->discount(m);
@@ -267,14 +268,10 @@ void SquareRootCLVModelTest::testSquareRootCLVMappingFunction() {
                 rTS->discount(m)).value();
 
             const CLVModelPayoff clvModelPayoff(
-                optionType, strike, boost::bind(g, t, _1));
+                optionType, strike,
+                ext::bind(g, t, ext::placeholders::_1));
 
-            const boost::function<Real(Real)> f =
-                boost::bind(std::multiplies<Real>(),
-                    boost::bind(
-                        &CLVModelPayoff::operator(), &clvModelPayoff, _1),
-                    boost::bind<Real>(boost::math::pdf<chi_squared_type, Real>,
-                        dist, _1) );
+            const ext::function<Real(Real)> f = integrand(clvModelPayoff, dist);
 
             const Array x = model.collocationPointsX(m);
             const Real calculated = GaussLobattoIntegral(1000, 1e-3)(
@@ -293,7 +290,7 @@ void SquareRootCLVModelTest::testSquareRootCLVMappingFunction() {
     }
 }
 
-namespace {
+namespace square_root_clv_model {
     class SquareRootCLVCalibrationFunction : public CostFunction {
       public:
         SquareRootCLVCalibrationFunction(
@@ -354,7 +351,7 @@ namespace {
                 bsProcess_, sqrtProcess, calibrationDates_,
                 14, 1-1e-14, 1e-14);
 
-            const boost::function<Real(Time, Real)> gSqrt = clvSqrtModel.g();
+            const ext::function<Real(Time, Real)> gSqrt = clvSqrtModel.g();
 
             Array retVal(resetDates_.size()*strikes_.size());
 
@@ -429,7 +426,7 @@ namespace {
                             strike, resetDate, payoff, exercise));
 
                     const Volatility implVol =
-                        detail::ImpliedVolatilityHelper::calculate(
+                        QuantLib::detail::ImpliedVolatilityHelper::calculate(
                             *fwdOption, *fwdEngine, *vol, npv, 1e-8, 200, 1e-4, 2.0);
 
                     const Size idx = k + i*strikes_.size();
@@ -437,10 +434,6 @@ namespace {
                 }
             }
 
-            Real s = 0.0;
-            for (Size i=0; i < retVal.size(); ++i)
-                s+=retVal[i]*retVal[i];
-
             return retVal;
         }
 
@@ -492,6 +485,8 @@ void SquareRootCLVModelTest::testForwardSkew() {
     BOOST_TEST_MESSAGE(
         "Testing forward skew dynamics with square root kernel process...");
 
+    using namespace square_root_clv_model;
+
     SavedSettings backup;
 
     const Date todaysDate(16, Oct, 2016);
@@ -560,7 +555,7 @@ void SquareRootCLVModelTest::testForwardSkew() {
             clvCalibrationDates.begin(), clvCalibrationDates.end()),
         14, 1-1e-14, 1e-14);
 
-    const boost::function<Real(Time, Real)> gSqrt = clvSqrtModel.g();
+    const ext::function<Real(Time, Real)> gSqrt = clvSqrtModel.g();
 
     const ext::shared_ptr<SimpleQuote> vol(
         ext::make_shared<SimpleQuote>(0.1));
@@ -685,7 +680,7 @@ void SquareRootCLVModelTest::testForwardSkew() {
                     strike, resetDate, payoff, exercise));
 
             const Volatility implVol =
-                detail::ImpliedVolatilityHelper::calculate(
+                QuantLib::detail::ImpliedVolatilityHelper::calculate(
                     *fwdOption, *fwdEngine, *vol, npv, 1e-8, 200, 1e-4, 2.0);
 
             const Size idx = j + i*LENGTH(strikes);
